﻿{*
 *  LRBarImg
 *
 *  各種バーコードを出力するアイテム
 *
 *  Copyright(c) 2010 seizo  All rights reserved.
 *  Copyright(c) 2010 DEKO   All rights reserved.
 *
 *  2010.10.18 初版作成
 *  2010.10.20 初期値等の調整
 *  2010.12.26 JPEG 対応
 *
 *}
unit LRBarImg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Clipbrd, jpeg, LightRep, LightRepEx;

type
  TBarTypes = (cbarJAN, cbarUPC, cbarCODE39, cbarNW7, cbarITF, cbarCTF, cbarIATA,
               cbarMATRIX, cbarNEC, cbarCUSTOMER, cbarCODE128, cbarCODE93);

  TPictures = (picBMP, picJPEG, picEMF, picWMF);

  TSCale = 1..10;

  TElementRatio = 20..30;

  TFrameSize = 0..100;

  TLocation = (locLeft, locCenter, locRight);

  { TLRBarImage クラス }
  {$IF CompilerVersion >= 23.0}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
  {$IFEND}
  TLRBarImage = class(TLRImage)
  private
    { Private 宣言 }
    FBarType: TBarTypes;
    FCode: String;
    FCommentUp: String;
    FCommentUpLoc: TLocation;
    FCommentDown: String;
    FCommentDownLoc: TLocation;
    FStartChar: Char;
    FStopChar: Char;
    FCD: Boolean;
    FCDN: Integer;
    FScale: TScale;
    FAngle: Integer;
    FReverse: Boolean;
    FSymbol: String;
    FSymbolDisp: Boolean;
    FBarFont: TFont;
    FTransparent: Boolean;
    FBarColor: TColor;
    FBackColor: TColor;
    FBarWidth: Integer;
    FBarHeight: Integer;
    FBarLeft: Integer;
    FBarTop: Integer;
    FBarSpaceUp: Integer;
    FBarSpaceDown: Integer;
    FBarSpaceLeft: Integer;
    FBarSpaceRight: Integer;
    FMatch: Boolean;
    FBarPicture: TPictures;
    FMfCanvas: TMetafileCanvas;   // メタファイル用キャンバス
    FElementRatio: TElementRatio;
    FBarEnabled: Boolean;
    FBarDebug: Boolean;           // デバッグ用プロパティ
    FBarDisped: Boolean;
    FBarFrame: TFrameSize;        // バーコードの周囲に描くフレーム(外枠)のサイズ
    function GetCDC: String;
    procedure SetBarType(const Value: TBarTypes);
    procedure SetCode(const Value: String);
    procedure SetCommentUp(const Value: String);
    procedure SetCommentUpLoc(const Value: TLocation);
    procedure SetCommentDown(const Value: String);
    procedure SetCommentDownLoc(const Value: TLocation);
    procedure SetStartChar(const Value: Char);
    procedure SetStopChar(const Value: Char);
    procedure SetCD(const Value: Boolean);
    procedure SetScale(const Value: TScale);
    procedure SetAngle(const Value: Integer);
    procedure SetReverse(const Value: Boolean);
    procedure SetSymbolDisp(const Value: Boolean);
    procedure SetBarFont(const Value: TFont);
    procedure SetTransparent(const Value: Boolean);
    procedure SetBarColor(const Value: TColor);
    procedure SetBackColor(const Value: TColor);
    procedure SetBarHeight(const Value: Integer);
    procedure SetBarLeft(const Value: Integer);
    procedure SetBarTop(const Value: Integer);
    procedure SetBarSpaceUp(const Value: Integer);
    procedure SetBarSpaceDown(const Value: Integer);
    procedure SetBarSpaceLeft(const Value: Integer);
    procedure SetBarSpaceRight(const Value: Integer);
    procedure SetMatch(const Value: Boolean);
    procedure SetBarPicture(const Value: TPictures);
    procedure SetElementRatio(const Value: TElementRatio);
    procedure SetBarEnabled(const Value: Boolean);
    procedure SetBarDebug(const Value: Boolean);
    procedure SetBarFrame(const Value: TFrameSize);
    procedure DrawCUSTOMER(const Position, Cnt, Scl, Style: Integer);
    function WidthCUSTOMER(const Module: String): Integer;
    procedure PaintBarcodeB;
    procedure PaintBarcodeM;
    function UPCE2A(const Str: String): String;
    procedure DrawFrame(const fc: TCanvas; const r: TRect);
    procedure RotateBitmap(Degree: Integer);
    procedure ReverseBitmap;
  protected
    { Protected 宣言 }
    function Make_JAN: String;
    function Make_UPC: String;
    function Make_CODE39: String;
    function Make_NW7: String;
    function Make_ITF: String;
    function Make_CTF: String;
    function Make_IATA: String;
    function Make_MATRIX: String;
    function Make_NEC: String;
    function Make_CUSTOMER: String;
    function Make_CODE128: String;
    function Make_CODE93: String;
    procedure PaintBarcode; virtual;
  public
    { Public 宣言 }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published 宣言(プロパティ宣言) }
    property Angle: Integer read FAngle write SetAngle default 0;
    property BackColor: TColor read FBackColor write SetBackColor default clWhite;
    property BarColor: TColor read FBarColor write SetBarColor default clBlack;
    property BarDebug: Boolean read FBarDebug write SetBarDebug nodefault;
    property BarDisped: Boolean read FBarDisped nodefault;
    property BarEnabled: Boolean read FBarEnabled write SetBarEnabled default True;
    property BarFont: TFont read FBarFont write SetBarFont stored True;
    property BarFrame: TFrameSize read FBarFrame write SetBarFrame default 0;
    property BarHeight: Integer read FBarHeight write SetBarHeight default 50;
    property BarLeft: Integer read FBarLeft write SetBarLeft nodefault;
    property BarPicture: TPictures read FBarPicture write SetBarPicture default picBMP;
    property BarSpaceDown: Integer read FBarSpaceDown write SetBarSpaceDown nodefault;
    property BarSpaceLeft: Integer read FBarSpaceLeft write SetBarSpaceLeft default 10;
    property BarSpaceRight: Integer read FBarSpaceRight write SetBarSpaceRight default 10;
    property BarSpaceUp: Integer read FBarSpaceUp write SetBarSpaceUp nodefault;
    property BarTop: Integer read FBarTop write SetBarTop nodefault;
    property BarType: TBarTypes read FBarType write SetBarType default cbarJAN;
    property BarWidth: Integer read FBarWidth nodefault;
    property CD: Boolean read FCD write SetCD nodefault;
    property CDC: String read GetCDC;
    property CDN: Integer read FCDN nodefault;
    property Code: String read FCode write SetCode nodefault;
    property CommentDown: String read FCommentDown write SetCommentDown nodefault;
    property CommentDownLoc: TLocation read FCommentDownLoc write SetCommentDownLoc nodefault;
    property CommentUp: String read FCommentUp write SetCommentUp nodefault;
    property CommentUpLoc: TLocation read FCommentUpLoc write SetCommentUpLoc nodefault;
    property ElementRatio: TElementRatio read FElementRatio write SetElementRatio default 20;
    property Match: Boolean read FMatch write SetMatch default True;
    property Reverse: Boolean read FReverse write SetReverse default False;
    property Scale: TScale read FScale write SetScale default 1;
    property StartChar: Char read FStartChar write SetStartChar default #64;
    property StopChar: Char read FStopChar write SetStopChar default #64;
    property Symbol: String read FSymbol nodefault;
    property SymbolDisp: Boolean read FSymbolDisp write SetSymbolDisp default True;
    property Transparent: Boolean read FTransparent write SetTransparent nodefault;
  end;

implementation

{ TLRBarImage }

constructor TLRBarImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FBarType := cbarJAN;
  FCode := '';
  FCommentUp := '';
  FCommentUpLoc := locLeft;
  FCommentDown := '';
  FCommentDownLoc := locLeft;
  FStartChar := Char('A');
  FStopChar := Char('A');
  FCD := False;
  FCDN := -1; // チェックデジットが存在しない場合の値
  FScale := 1;
  FAngle := 0;
  FReverse := False;
  FSymbol := '';
  FSymbolDisp := True;
  FBarFont := TFont.Create;
  FTransparent := False;
  FBarColor := clBlack;
  FBackColor := clWhite;
  FBarWidth := 0;
  FBarHeight := 50;
  FBarLeft := 0;
  FBarTop := 0;
  FBarSpaceUp := 0;
  FBarSpaceDown := 0;
  FBarSpaceLeft := 10;
  FBarSpaceRight := 10;
  FMatch := True;
  FBarPicture := picBMP;
  FElementRatio := 20;
  FBarEnabled := True;
  FBarDebug := False;
  FBarDisped := False;
  FBarFrame := 0;
end;

destructor TLRBarImage.Destroy;
begin
  FBarFont.Free;
  inherited Destroy;
end;

function TLRBarImage.Make_JAN: String;
type
  CharData = array [0..9] of String;
var
  ChDCount,
  CodeType,
  pf      :Integer;
  i       ,
  j       ,
  k       :LongInt;
  Dmy     ,
  tb      ,
  Module  :String;
  pt      ,
  ol      ,
  el      ,
  er      :CharData;
begin
  tb := '0123456789';
  pt[0] := '111111';
  pt[1] := '110100';
  pt[2] := '110010';
  pt[3] := '110001';
  pt[4] := '101100';
  pt[5] := '100110';
  pt[6] := '100011';
  pt[7] := '101010';
  pt[8] := '101001';
  pt[9] := '100101';

  ol[0] := '0001101';
  ol[1] := '0011001';
  ol[2] := '0010011';
  ol[3] := '0111101';
  ol[4] := '0100011';
  ol[5] := '0110001';
  ol[6] := '0101111';
  ol[7] := '0111011';
  ol[8] := '0110111';
  ol[9] := '0001011';

  el[0] := '0100111';
  el[1] := '0110011';
  el[2] := '0011011';
  el[3] := '0100001';
  el[4] := '0011101';
  el[5] := '0111001';
  el[6] := '0000101';
  el[7] := '0010001';
  el[8] := '0001001';
  el[9] := '0010111';

  er[0] := '1110010';
  er[1] := '1100110';
  er[2] := '1101100';
  er[3] := '1000010';
  er[4] := '1011100';
  er[5] := '1001110';
  er[6] := '1010000';
  er[7] := '1000100';
  er[8] := '1001000';
  er[9] := '1110100';

  Dmy := FCode;

  case Length(Dmy) of
    7, 8:
      begin
        CodeType := 0;
        Dmy := Copy(Dmy, 1,  7);
      end;
    12, 13:
      begin
        CodeType := 1;
        Dmy := Copy(Dmy, 1, 12);
      end;
  else
    Result := '';
    Exit;
  end;

  j := 0;
  for i := 1 to Length(Dmy) do
    begin
      if Pos(Dmy[i], tb) = 0 then
        begin
          j := i;
          Break;
        end;
    end;
  if j <> 0 then
    begin
      Result := '';
      Exit;
    end;

  {チェックデジットの付加(自動生成) }
  // 2000/03/10 Update M&I From Here 
  j := 0;
  ChDCount := 1;
  for i:= Length(Dmy) downto 1 do
    begin
      if (ChDCount mod 2) = 1 then
       j := j + StrToInt(Dmy[i]) * 3
      else
       j := j + StrToInt(Dmy[i]);
      Inc(ChDCount);
    end;

  j := j mod 10;
  if j > 0 then
    j := 10 - j;
  // 2000/03/10 Update M&I Until Here 

  Dmy := Dmy + IntToStr(j);

  {バーコードの左側の作成}
  Module := '101';
  pf := StrToInt('0' + Dmy[1]);
  for i := (CodeType + 1) to (CodeType * 3 + 4) do
    begin
      k := StrToInt('0' + Dmy[i]);
      if (CodeType = 0) or (Copy(pt[pf],i-1,1) = '1') then
        Module := Module + ol[k]
      else
        Module := Module + el[k];
    end;

  {バーコードの中央の作成}
  Module := Module + '01010';

  {バーコードの右側の作成}
  for i:=(CodeType * 3 + 5) to (CodeType * 5 + 8) do
    begin
      k := StrToInt('0' + Dmy[i]);
      Module := Module + er[k]
    end;
  Module := Module + '1010';

  FSymbol := Dmy;
  FCDN := j;
  Result := Module;
end;

function TLRBarImage.Make_UPC: String;
type
  CharData = array[0..9] of String;

var
  ChDCount,
  CodeType,
  pf      : Integer;
  i       ,
  j       ,
  k       : LongInt;
  Dmy     ,
  tb      ,
  Module  : String;
  pt      ,
  ol      ,
  el      ,
  er      : CharData;

begin
  tb := '0123456789';
  pt[0] := '111111';
  pt[1] := '110100';
  pt[2] := '110010';
  pt[3] := '110001';
  pt[4] := '101100';
  pt[5] := '100110';
  pt[6] := '100011';
  pt[7] := '101010';
  pt[8] := '101001';
  pt[9] := '100101';

  ol[0] := '0001101';
  ol[1] := '0011001';
  ol[2] := '0010011';
  ol[3] := '0111101';
  ol[4] := '0100011';
  ol[5] := '0110001';
  ol[6] := '0101111';
  ol[7] := '0111011';
  ol[8] := '0110111';
  ol[9] := '0001011';

  el[0] := '0100111';
  el[1] := '0110011';
  el[2] := '0011011';
  el[3] := '0100001';
  el[4] := '0011101';
  el[5] := '0111001';
  el[6] := '0000101';
  el[7] := '0010001';
  el[8] := '0001001';
  el[9] := '0010111';

  er[0] := '1110010';
  er[1] := '1100110';
  er[2] := '1101100';
  er[3] := '1000010';
  er[4] := '1011100';
  er[5] := '1001110';
  er[6] := '1010000';
  er[7] := '1000100';
  er[8] := '1001000';
  er[9] := '1110100';

  Dmy := FCode;

  case Length(Dmy) of
    7, 8:
      begin
        CodeType := 0;
        Dmy := Copy(Dmy, 1,  7);
      end;
    11, 12:
      begin
        CodeType := 1;
        Dmy := Copy(Dmy, 1, 11);
      end;
  else
    Result := '';
    Exit;
  end;

  j := 0;
  for i := 1 to Length(Dmy) do
  begin
    if Pos(Dmy[i], tb) = 0 then
    begin
      j := i;
      Break;
    end;
  end;
  if j <> 0 then
  begin
    Result := '';
    Exit;
  end;

  if (CodeType = 0) then
    Module := UPCE2A(Dmy)
  else
    Module := Dmy;

  if (Module = '') then
  begin
    Result := '';
    Exit;
  end;

  { チェックデジットの付加(自動生成) }
  j := 0;
  ChDCount := 1;
  for i:= Length(Module) downto 1 do
  begin
    if (ChDCount mod 2) = 1 then
     j := j + StrToInt(Module[i]) * 3
    else
     j := j + StrToInt(Module[i]);
    Inc(ChDCount);
  end;

  j := j mod 10;
  if j > 0 then
    j := 10 - j;

  Dmy := Dmy + IntToStr(j);

  { 各バーコードの作成 }
  if CodeType = 1 then
  begin
    { バーコードの左側の作成 }
    Module := '101';
    k := StrToInt('0' + Dmy[1]);
    Module := Module + ol[k];               // ナンバーシステム
    for i := (CodeType + 1) to (CodeType * 3 + 3) do
    begin
      k := StrToInt('0' + Dmy[i]);
      Module := Module + ol[k]
    end;

    { バーコードの中央の作成 }
    Module := Module + '01010';

    { バーコードの右側の作成 }
    for i:=(CodeType * 3 + 4) to (CodeType * 5 + 6) do
    begin
      k := StrToInt('0' + Dmy[i]);
      Module := Module + er[k]
    end;
    k := StrToInt('0' + Dmy[12]);
    Module := Module + er[k];               // チェックデジット
    Module := Module + '1010';
    end
  else
  begin
    Module := '101';
    pf := StrToInt('0' + Dmy[8]); // チェックデジットは、パリティで表現する。
    for i := 2 to 7 do
    begin
      k := StrToInt('0' + Dmy[i]);
      if (Copy(pt[pf], i - 1, 1) = '1') then
        Module := Module + el[k]
      else
        Module := Module + ol[k];
    end;
    Module := Module + '0101010';
  end;

  FSymbol := Dmy;
  FCDN := j;
  Result := Module;
end;

function TLRBarImage.Make_CODE39: String;
var
  i    ,
  j    :Integer;
  Check:LongInt;
  Dmy  ,
  pm   ,
  tb   :String;
  bp   :array [$00..$2B] of String;
begin
  tb := '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ-. $/+%';
  bp[$00] := '1012303010'; // 0
  bp[$01] := '3012101030'; // 1
  bp[$02] := '1032101030'; // 2
  bp[$03] := '3032101010'; // 3
  bp[$04] := '1012301030'; // 4
  bp[$05] := '3012301010'; // 5
  bp[$06] := '1032301010'; // 6
  bp[$07] := '1012103030'; // 7
  bp[$08] := '3012103010'; // 8
  bp[$09] := '1032103010'; // 9
  bp[$0A] := '3010121030'; // A
  bp[$0B] := '1030121030'; // B
  bp[$0C] := '3030121010'; // C
  bp[$0D] := '1010321030'; // D
  bp[$0E] := '3010321010'; // E
  bp[$0F] := '1030321010'; // F
  bp[$10] := '1010123030'; // G
  bp[$11] := '3010123010'; // H
  bp[$12] := '1030123010'; // I
  bp[$13] := '1010323010'; // J
  bp[$14] := '3010101230'; // K
  bp[$15] := '1030101230'; // L
  bp[$16] := '3030101210'; // M
  bp[$17] := '1010301230'; // N
  bp[$18] := '3010301210'; // O
  bp[$19] := '1030301210'; // P
  bp[$1A] := '1010103230'; // Q
  bp[$1B] := '3010103210'; // R
  bp[$1C] := '1030103210'; // S
  bp[$1D] := '1010303210'; // T
  bp[$1E] := '3210101030'; // U
  bp[$1F] := '1230101030'; // V
  bp[$20] := '3230101010'; // W
  bp[$21] := '1210301030'; // X
  bp[$22] := '3210301010'; // Y
  bp[$23] := '1230301010'; // Z
  bp[$24] := '1210103030'; // -
  bp[$25] := '3210103010'; // .
  bp[$26] := '1230103010'; // [SPACE]
  bp[$27] := '1212121010'; // $
  bp[$28] := '1212101210'; // /
  bp[$29] := '1210121210'; // +
  bp[$2A] := '1012121210'; // %
  bp[$2B] := '1210303010'; // *

  Dmy := FCode;

  if Dmy = '' then
    begin
      Result := '';
      Exit;
    end;

  j := 0;
  for i := 1 to Length(Dmy) do
    begin
      if Pos(Dmy[i], tb) = 0 then
        begin
          j := i;
          Break;
        end;
    end;
  if j <> 0 then
    begin
      Result := '';
      Exit;
    end;

  {チェックデジットの算出}
  Check := 0;
  if FCD then
    begin
      // すべての文字を与えられたコードに当てはめて加算
      for i:=1 to Length(Dmy) do
//      Check := Check + Pos(Dmy[i],tb);      // 2000/02/21 Delete DEKO
        Check := Check + Pos(Dmy[i],tb) - 1;  // 2000/02/21 Modify M&I
      // モジュラス４３でチェックデジットを算出
      Check := Check mod 43;
    end;

  {バーコードの作成}
  pm := '';
  pm := pm + bp[$2B];                      // スタートキャラクタを'*'追加
  for i:=1 to Length(Dmy) do
    pm := pm + bp[Pos(Dmy[i],tb)-1];
  if FCD then                               // チェックディジットを追加
    pm := pm + bp[Check];
  pm := pm + bp[$2B];                      // ストップキャラクタ'*'を追加

  FSymbol := '*' + Dmy;
  if FCD then
    FSymbol := FSymbol + tb[Check + 1];
  FSymbol := FSymbol + '*';
  if FCD then
    FCDN := Check;
  Result := pm;
end;

function TLRBarImage.Make_NW7: String;
var
  i    ,
  j    :Integer;
  Check:LongInt;
  Dmy  ,
  pm   ,
  tb   :String;
  bp   :array [$00..$13] of String;
begin
  tb := '0123456789-$:/.+ABCD';
  bp[$00] := '10101230'; // 0
  bp[$01] := '10103210'; // 1
  bp[$02] := '10121030'; // 2
  bp[$03] := '32101010'; // 3
  bp[$04] := '10301210'; // 4
  bp[$05] := '30101210'; // 5
  bp[$06] := '12101030'; // 6
  bp[$07] := '12103010'; // 7
  bp[$08] := '12301010'; // 8
  bp[$09] := '30121010'; // 9
  bp[$0A] := '10123010'; // -
  bp[$0B] := '10321010'; // $
  bp[$0C] := '30103030'; // :
  bp[$0D] := '30301030'; // /
  bp[$0E] := '30303010'; // .
  bp[$0F] := '10303030'; // +
  bp[$10] := '10321210'; // A
  bp[$11] := '12121030'; // B
  bp[$12] := '10121230'; // C
  bp[$13] := '10123210'; // D

  Dmy := FCode;

  if Dmy = '' then
    begin
      Result := '';
      Exit;
    end;

  j := 0;
  for i := 1 to Length(Dmy) do
    begin
      if Pos(Dmy[i], tb) = 0 then
        begin
          j := i;
          Break;
        end;
    end;
  if j <> 0 then
    begin
      Result := '';
      Exit;
    end;

  {チェックデジットの算出}
  Check := 0;
  if FCD then
    begin
      // すべての文字を与えられたコードに当てはめて加算
      Check := Check + Pos(FStartChar,tb) - 1; // 2001/09/23 Modify seizo
      for i:=1 to Length(Dmy) do
//      Check := Check + Pos(Dmy[i],tb);       // 2000/02/21 Delete DEKO
        Check := Check + Pos(Dmy[i],tb) - 1;   // 2000/02/21 Modify M&I
      Check := Check + Pos(FStopChar,tb) - 1;  // 2001/09/23 Modify seizo
      // モジュラス16でチェックデジットを算出
      if (Check mod 16) = 0 then
        Check := 0
      else
        Check := 16 - (Check mod 16);
    end;

  {バーコードの作成}
  pm := '';
  pm := pm + bp[Pos(FStartChar,tb)-1];      // スタートキャラクタを追加
  for i:=1 to Length(Dmy) do
    pm := pm + bp[Pos(Dmy[i],tb)-1];
  if FCD then                               // チェックディジットを追加
    pm := pm + bp[Check];
  pm := pm + bp[Pos(FStopChar,tb)-1];       // ストップキャラクタを追加

  FSymbol := FStartChar + Dmy;
  if FCD then
    FSymbol := FSymbol + tb[Check + 1];
  FSymbol := FSymbol + FStopChar;
  if FCD then
    FCDN := Check;
  Result := pm;
end;

function TLRBarImage.Make_ITF: String;
type
  CharData = array[0..9] of String;
var
  len,
  Check,
  ChDCount:Integer;
  i       ,
  j	  ,
  k       :LongInt;
  Dmy     ,
  tb      ,
  ocd	  ,
  ecd	  ,
  Module  :String;
  oc      ,
  ec      :CharData;
  se      :array[0..1] of String;
begin
  Result := '';

  tb := '0123456789';

  oc[0] := '1 1 3 3 1 ';
  oc[1] := '3 1 1 1 3 ';
  oc[2] := '1 3 1 1 3 ';
  oc[3] := '3 3 1 1 1 ';
  oc[4] := '1 1 3 1 3 ';
  oc[5] := '3 1 3 1 1 ';
  oc[6] := '1 3 3 1 1 ';
  oc[7] := '1 1 1 3 3 ';
  oc[8] := '3 1 1 3 1 ';
  oc[9] := '1 3 1 3 1 ';

  ec[0] := '0 0 2 2 0';
  ec[1] := '2 0 0 0 2';
  ec[2] := '0 2 0 0 2';
  ec[3] := '2 2 0 0 0';
  ec[4] := '0 0 2 0 2';
  ec[5] := '2 0 2 0 0';
  ec[6] := '0 2 2 0 0';
  ec[7] := '0 0 0 2 2';
  ec[8] := '2 0 0 2 0';
  ec[9] := '0 2 0 2 0';

  se[0] := '1010';
  se[1] := '301';
  
  Dmy := FCode;

  if Dmy = '' then
    begin
      Result := '';
      Exit;
    end;

  j := 0;
  for i := 1 to Length(Dmy) do
    begin
      if Pos(Dmy[i], tb) = 0 then
        begin
          j := i;
          Break;
        end;
    end;
  if j <> 0 then
    begin
      Result := '';
      Exit;
    end;

  len := Length(Dmy);
  if FCD = True then
    len := len + 1;
  if (len mod 2) = 1 then  // 奇数桁の場合
    Dmy := '0' + Dmy;      // 先頭に'0'を付加して偶数桁にする。

  {チェックデジットの付加 }
  Check := 0;
  if FCD then
    begin
      ChDCount := 1;
      for i:= Length(Dmy) downto 1 do
        begin
          if (ChDCount mod 2) = 1 then
            Check := Check + StrToInt(Dmy[i]) * 3
          else
            Check := Check + StrToInt(Dmy[i]);
          Inc(ChDCount);
        end;
      // モジュラス１０でチェックデジットを算出
      Check := Check mod 10;
      // 0の場合はそのまま
      if Check > 0 then
        Check := 10 - Check;

      Dmy := Dmy + IntToStr(Check);
    end;
    
  Module := se[0];
  for i := 0 to Length(Dmy) div 2 - 1 do
    begin
      // 奇数
      k := StrToInt('0' + Dmy[i * 2 + 1]);
      ocd := oc[k];
      // 偶数
      k := StrToInt('0' + Dmy[i * 2 + 2]);
      ecd := ec[k];
      // 奇数ｺｰﾄﾞと偶数ｺｰﾄﾞを組合わせる
      for j := 1 to 9 do
        begin
          k := j mod 2;
          if k = 1 then
            ocd[j + 1] := ecd[j];
        end;
      Module := Module + ocd;
    end;
  Module := Module + se[1];

  FSymbol := Dmy;
  if FCD then
    FCDN := Check;
  Result := Module;
end;

function TLRBarImage.Make_CTF: String;
var
  i       ,
  j       ,
  Check   ,
  ChDCount: Integer;
  Dmy     ,
  Module  ,
  tb      : String;
  bp      : array [$00..$09] of String;
  se      : array[0..1] of String;

begin
  tb := '0123456789';
  bp[$00] := '1010303010'; // 0
  bp[$01] := '3010101030'; // 1
  bp[$02] := '1030101030'; // 2
  bp[$03] := '3030101010'; // 3
  bp[$04] := '1010301030'; // 4
  bp[$05] := '3010301010'; // 5
  bp[$06] := '1030301010'; // 6
  bp[$07] := '1010103030'; // 7
  bp[$08] := '3010103010'; // 8
  bp[$09] := '1030103010'; // 9

  { スタート・ストップキャラクタ }
  se[0] := '303010';
  se[1] := '30103';

  Dmy := FCode;

  if Dmy = '' then
  begin
    Result := '';
    Exit;
  end;

  j := 0;
  for i := 1 to Length(Dmy) do
  begin
    if Pos(Dmy[i], tb) = 0 then
    begin
      j := i;
      Break;
    end;
  end;
  if j <> 0 then
  begin
    Result := '';
    Exit;
  end;

  { チェックデジットの付加 }
  Check := 0;
  if FCD then
  begin
    ChDCount := 1;
    for i:= Length(Dmy) downto 1 do
    begin
      if (ChDCount mod 2) = 1 then
        Check := Check + StrToInt(Dmy[i]) * 3
      else
        Check := Check + StrToInt(Dmy[i]);
      Inc(ChDCount);
    end;
    // モジュラス１０でチェックデジットを算出
    Check := Check mod 10;
    // 0の場合はそのまま
    if Check > 0 then
      Check := 10 - Check;

    Dmy := Dmy + IntToStr(Check);
  end;

  { バーコードの作成 }
  Module := se[0];                   // スタートキャラクタ
  for i:=1 to Length(Dmy) do
    Module := Module + bp[Pos(Dmy[i], tb) - 1];
  Module := Module + se[1];          // ストップキャラクタ

  FSymbol := Dmy;
  if FCD then
    FCDN := Check;
  Result := Module;
end;

function TLRBarImage.Make_IATA: String;
var
  i       ,
  j       ,
  Check   ,
  ChDCount: Integer;
  Dmy     ,
  Module  ,
  tb      : String;
  bp      : array [$00..$09] of String;
  se      : array[0..1] of String;

begin
  tb := '0123456789';
  bp[$00] := '1010303010'; // 0
  bp[$01] := '3010101030'; // 1
  bp[$02] := '1030101030'; // 2
  bp[$03] := '3030101010'; // 3
  bp[$04] := '1010301030'; // 4
  bp[$05] := '3010301010'; // 5
  bp[$06] := '1030301010'; // 6
  bp[$07] := '1010103030'; // 7
  bp[$08] := '3010103010'; // 8
  bp[$09] := '1030103010'; // 9

  { スタート・ストップキャラクタ }
  se[0] := '1010';
  se[1] := '301';

  Dmy := FCode;

  if Dmy = '' then
  begin
    Result := '';
    Exit;
  end;

  j := 0;
  for i := 1 to Length(Dmy) do
  begin
    if Pos(Dmy[i], tb) = 0 then
    begin
      j := i;
      Break;
    end;
  end;
  if j <> 0 then
  begin
    Result := '';
    Exit;
  end;

  { チェックデジットの付加(自動生成) }
  Check := 0;
  ChDCount := 1;
  for i:= Length(Dmy) downto 1 do
  begin
    if (ChDCount mod 2) = 1 then
      Check := Check + StrToInt(Dmy[i]) * 3
    else
      Check := Check + StrToInt(Dmy[i]);
    Inc(ChDCount);
  end;
  // モジュラス１０でチェックデジットを算出
  Check := Check mod 10;
  // 0の場合はそのまま
  if Check > 0 then
    Check := 10 - Check;

  Dmy := Dmy + IntToStr(Check);

  { バーコードの作成 }
  Module := se[0];                   // スタートキャラクタ
  for i:=1 to Length(Dmy) do
    Module := Module + bp[Pos(Dmy[i], tb) - 1];
  Module := Module + se[1];          // ストップキャラクタ

  FSymbol := Dmy;
  FCDN := Check;
  Result := Module;
end;

function TLRBarImage.Make_MATRIX: String;
var
  i       ,
  j       ,
  Check   ,
  ChDCount: Integer;
  Dmy     ,
  Module  ,
  tb      : String;
  bp      : array [$00..$09] of String;
  se      : array[0..1] of String;

begin
  tb := '0123456789';
  bp[$00] := '103210'; // 0
  bp[$01] := '301030'; // 1
  bp[$02] := '121030'; // 2
  bp[$03] := '321010'; // 3
  bp[$04] := '103030'; // 4
  bp[$05] := '303010'; // 5
  bp[$06] := '123010'; // 6
  bp[$07] := '101230'; // 7
  bp[$08] := '301210'; // 8
  bp[$09] := '121210'; // 9

  { スタート・ストップキャラクタ }
  se[0] := '301010';
  se[1] := '30101';

  Dmy := FCode;

  if Dmy = '' then
  begin
    Result := '';
    Exit;
  end;

  j := 0;
  for i := 1 to Length(Dmy) do
  begin
    if Pos(Dmy[i], tb) = 0 then
    begin
      j := i;
      Break;
    end;
  end;
  if j <> 0 then
  begin
    Result := '';
    Exit;
  end;

  { チェックデジットの付加 }
  Check := 0;
  if FCD then
  begin
    ChDCount := 1;
    for i:= Length(Dmy) downto 1 do
    begin
      if (ChDCount mod 2) = 1 then
        Check := Check + StrToInt(Dmy[i]) * 3
      else
        Check := Check + StrToInt(Dmy[i]);
      Inc(ChDCount);
    end;
    // モジュラス１０でチェックデジットを算出
    Check := Check mod 10;
    // 0の場合はそのまま
    if Check > 0 then
      Check := 10 - Check;

    Dmy := Dmy + IntToStr(Check);
  end;

  { バーコードの作成 }
  Module := se[0];                    // スタートキャラクタ
  for i:=1 to Length(Dmy) do
    Module := Module + bp[Pos(Dmy[i], tb) - 1];
  Module := Module + se[1];           // ストップキャラクタ

  FSymbol := Dmy;
  if FCD then
    FCDN := Check;
  Result := Module;
end;

function TLRBarImage.Make_NEC: String;
var
  i       ,
  j       ,
  Check   ,
  ChDCount: Integer;
  Dmy     ,
  Module  ,
  tb      : String;
  bp      : array [$00..$09] of String;
  se      : array[0..1] of String;

begin
  tb := '0123456789';
  bp[$00] := '321010'; // 0
  bp[$01] := '101230'; // 1
  bp[$02] := '103030'; // 2
  bp[$03] := '103210'; // 3
  bp[$04] := '121030'; // 4
  bp[$05] := '121210'; // 5
  bp[$06] := '123010'; // 6
  bp[$07] := '301030'; // 7
  bp[$08] := '301210'; // 8
  bp[$09] := '303010'; // 9

  { スタート・ストップキャラクタ }
  se[0] := '3030';
  se[1] := '123';

  Dmy := FCode;

  if Dmy = '' then
  begin
    Result := '';
    Exit;
  end;

  j := 0;
  for i := 1 to Length(Dmy) do
  begin
    if Pos(Dmy[i], tb) = 0 then
    begin
      j := i;
      Break;
    end;
  end;
  if j <> 0 then
  begin
    Result := '';
    Exit;
  end;

  { チェックデジットの付加 }
  Check := 0;
  if FCD then
  begin
    ChDCount := 1;
    for i:= Length(Dmy) downto 1 do
    begin
      if (ChDCount mod 2) = 1 then
        Check := Check + StrToInt(Dmy[i]) * 3
      else
        Check := Check + StrToInt(Dmy[i]);
      Inc(ChDCount);
    end;
    // モジュラス１０でチェックデジットを算出
    Check := Check mod 10;
    // 0の場合はそのまま
    if Check > 0 then
      Check := 10 - Check;

    Dmy := Dmy + IntToStr(Check);
  end;

  { バーコードの作成 }
  Module := se[0];                  // スタートキャラクタ
  for i:=1 to Length(Dmy) do
    Module := Module + bp[Pos(Dmy[i], tb) - 1];
  Module := Module + se[1];         // ストップキャラクタ

  FSymbol := Dmy;
  if FCD then
    FCDN := Check;
  Result := Module;
end;

function TLRBarImage.Make_CUSTOMER: String;
var
  ch,
  ch2:Byte;
  i,j,k:Integer;
  bc:array[0..10] of String;
  sc:array[0.. 7] of String;
  se:array[0.. 1] of String;
  Dmy,
  tb,
  Module:String;
begin
  // ０２３６７
  // □□□■■　　
  // □■■■■
  // □□■□■　
  tb := '0123456789-        ';
  bc[ 0] := '702020'; // 0
  bc[ 1] := '707020'; // 1
  bc[ 2] := '703060'; // 2
  bc[ 3] := '307060'; // 3
  bc[ 4] := '706030'; // 4
  bc[ 5] := '702070'; // 5
  bc[ 6] := '306070'; // 6
  bc[ 7] := '607030'; // 7
  bc[ 8] := '603070'; // 8
  bc[ 9] := '207070'; // 9
  bc[10] := '207020'; // -

  sc[0]  := '306020';
  sc[1]  := '302060';
  sc[2]  := '603020';
  sc[3]  := '203060';
  sc[4]  := '602030';
  sc[5]  := '206030';
  sc[6]  := '202070';
  sc[7]  := '707070';

  se[0]  := '007030'; // Start
  se[1]  := '307000'; // End

  Dmy := FCode + StringOfChar(' ', 21 - Length(FCode));

  // Start
  Module := se[0];
  j := 0;
  k := 0;
  for i:=1 to 20 do
    begin
      ch := Ord(Dmy[i]);
      case ch of
        // -,0..9
        45,
        48..57:
          begin
            if ch  = 45 then
              ch2 := 10
            else
              ch2 := ch-48;
            Module := Module + bc[ch2];
            Inc(j,ch2);
            Inc(k);
          end;
        // ' ',A..Z
        32,
        65..90:
          begin
            if (ch = 32) then
              begin
                Module := Module + sc[3];
                Inc(j,14);
                Inc(k);
              end
            else
              begin
                if k < 19 then
                  begin
                    ch2 := ((ch - 65) div 10);
                    Module := Module + sc[ch2];
                    Inc(j,ch2 + 11);
                    Inc(k);
                    ch2 := ((ch - 65) mod 10);
                    Module := Module + bc[ch2];
                    Inc(j,ch2);
                    Inc(k);
                  end
                else
                  begin
                    if (k <= 19) then
                      begin
                        Module := Module + sc[3];
                        Inc(j,14);
                        Inc(k);
                      end;
                  end;
              end;
          end;
        else
      end;
      if (k > 19) then
        Break;
    end;

  // チェックデジット
  if (j mod 19) = 0 then
    j := 0
  else
    j := (19 - (j mod 19));
  case j of
    0..10:
      Module := Module + bc[j];
    11..18:
      Module := Module + sc[j-11];
  end;
  // End
  Module := Module + se[1];

  FSymbol := Dmy + tb[j + 1];
  FCDN := j;
  Result := Module;
end;

function TLRBarImage.Make_CODE128: String;
var
  sc:array [$00..$66,0..3] of String;
  se:array [0..3] of String;
  EndCmd:Boolean;
  Command,
  Dmy,
  Module:String;
  i,
  SCC,
  Code,
  ShiftCode,
  Index,
  Count:Integer;
  function SearchCode(SCC:Byte;Cmd:String):Integer;
  var
    i:Integer;
  begin
    Result := -1;
    for i:=$00 to $66 do
      begin
        if sc[i][SCC] = Cmd then
          begin
            Result := i;
            Break;
          end;
      end;
  end;
begin
  sc[$00][0] := ' ';       sc[$00][1] := ' ';      sc[$00][2] := '00';    sc[$00][3] := '303232';
  sc[$01][0] := '!';       sc[$01][1] := '!';      sc[$01][2] := '01';    sc[$01][3] := '323032';
  sc[$02][0] := '"';       sc[$02][1] := '"';      sc[$02][2] := '02';    sc[$02][3] := '323230';
  sc[$03][0] := '#';       sc[$03][1] := '#';      sc[$03][2] := '03';    sc[$03][3] := '121234';
  sc[$04][0] := '$';       sc[$04][1] := '$';      sc[$04][2] := '04';    sc[$04][3] := '121432';
  sc[$05][0] := '%';       sc[$05][1] := '%';      sc[$05][2] := '05';    sc[$05][3] := '141232';
  sc[$06][0] := '&';       sc[$06][1] := '&';      sc[$06][2] := '06';    sc[$06][3] := '123214';
  sc[$07][0] := '''';      sc[$07][1] := '''';     sc[$07][2] := '07';    sc[$07][3] := '123412';
  sc[$08][0] := '(';       sc[$08][1] := '(';      sc[$08][2] := '08';    sc[$08][3] := '143212';
  sc[$09][0] := ')';       sc[$09][1] := ')';      sc[$09][2] := '09';    sc[$09][3] := '321214';
  sc[$0A][0] := '*';       sc[$0A][1] := '*';      sc[$0A][2] := '10';    sc[$0A][3] := '321412';
  sc[$0B][0] := '+';       sc[$0B][1] := '+';      sc[$0B][2] := '11';    sc[$0B][3] := '341212';
  sc[$0C][0] := ',';       sc[$0C][1] := ',';      sc[$0C][2] := '12';    sc[$0C][3] := '103252';
  sc[$0D][0] := '-';       sc[$0D][1] := '-';      sc[$0D][2] := '13';    sc[$0D][3] := '123052';
  sc[$0E][0] := '.';       sc[$0E][1] := '.';      sc[$0E][2] := '14';    sc[$0E][3] := '123250';
  sc[$0F][0] := '/';       sc[$0F][1] := '/';      sc[$0F][2] := '15';    sc[$0F][3] := '105232';
  sc[$10][0] := '0';       sc[$10][1] := '0';      sc[$10][2] := '16';    sc[$10][3] := '125032';
  sc[$11][0] := '1';       sc[$11][1] := '1';      sc[$11][2] := '17';    sc[$11][3] := '125230';
  sc[$12][0] := '2';       sc[$12][1] := '2';      sc[$12][2] := '18';    sc[$12][3] := '325210';
  sc[$13][0] := '3';       sc[$13][1] := '3';      sc[$13][2] := '19';    sc[$13][3] := '321052';
  sc[$14][0] := '4';       sc[$14][1] := '4';      sc[$14][2] := '20';    sc[$14][3] := '321250';
  sc[$15][0] := '5';       sc[$15][1] := '5';      sc[$15][2] := '21';    sc[$15][3] := '305212';
  sc[$16][0] := '6';       sc[$16][1] := '6';      sc[$16][2] := '22';    sc[$16][3] := '325012';
  sc[$17][0] := '7';       sc[$17][1] := '7';      sc[$17][2] := '23';    sc[$17][3] := '503050';
  sc[$18][0] := '8';       sc[$18][1] := '8';      sc[$18][2] := '24';    sc[$18][3] := '501232';
  sc[$19][0] := '9';       sc[$19][1] := '9';      sc[$19][2] := '25';    sc[$19][3] := '521032';
  sc[$1A][0] := ':';       sc[$1A][1] := ':';      sc[$1A][2] := '26';    sc[$1A][3] := '521230';
  sc[$1B][0] := ';';       sc[$1B][1] := ';';      sc[$1B][2] := '27';    sc[$1B][3] := '503212';
  sc[$1C][0] := '<';       sc[$1C][1] := '<';      sc[$1C][2] := '28';    sc[$1C][3] := '523012';
  sc[$1D][0] := '=';       sc[$1D][1] := '=';      sc[$1D][2] := '29';    sc[$1D][3] := '523210';
  sc[$1E][0] := '>';       sc[$1E][1] := '>';      sc[$1E][2] := '30';    sc[$1E][3] := '303034';
  sc[$1F][0] := '?';       sc[$1F][1] := '?';      sc[$1F][2] := '31';    sc[$1F][3] := '303430';
  sc[$20][0] := '@';       sc[$20][1] := '@';      sc[$20][2] := '32';    sc[$20][3] := '343030';
  sc[$21][0] := 'A';       sc[$21][1] := 'A';      sc[$21][2] := '33';    sc[$21][3] := '101434';
  sc[$22][0] := 'B';       sc[$22][1] := 'B';      sc[$22][2] := '34';    sc[$22][3] := '141034';
  sc[$23][0] := 'C';       sc[$23][1] := 'C';      sc[$23][2] := '35';    sc[$23][3] := '141430';
  sc[$24][0] := 'D';       sc[$24][1] := 'D';      sc[$24][2] := '36';    sc[$24][3] := '103414';
  sc[$25][0] := 'E';       sc[$25][1] := 'E';      sc[$25][2] := '37';    sc[$25][3] := '143014';
  sc[$26][0] := 'F';       sc[$26][1] := 'F';      sc[$26][2] := '38';    sc[$26][3] := '143410';
  sc[$27][0] := 'G';       sc[$27][1] := 'G';      sc[$27][2] := '39';    sc[$27][3] := '301414';
  sc[$28][0] := 'H';       sc[$28][1] := 'H';      sc[$28][2] := '40';    sc[$28][3] := '341014';
  sc[$29][0] := 'I';       sc[$29][1] := 'I';      sc[$29][2] := '41';    sc[$29][3] := '341410';
  sc[$2A][0] := 'J';       sc[$2A][1] := 'J';      sc[$2A][2] := '42';    sc[$2A][3] := '103054';
  sc[$2B][0] := 'K';       sc[$2B][1] := 'K';      sc[$2B][2] := '43';    sc[$2B][3] := '103450';
  sc[$2C][0] := 'L';       sc[$2C][1] := 'L';      sc[$2C][2] := '44';    sc[$2C][3] := '143050';
  sc[$2D][0] := 'M';       sc[$2D][1] := 'M';      sc[$2D][2] := '45';    sc[$2D][3] := '105034';
  sc[$2E][0] := 'N';       sc[$2E][1] := 'N';      sc[$2E][2] := '46';    sc[$2E][3] := '105430';
  sc[$2F][0] := 'O';       sc[$2F][1] := 'O';      sc[$2F][2] := '47';    sc[$2F][3] := '145030';
  sc[$30][0] := 'P';       sc[$30][1] := 'P';      sc[$30][2] := '48';    sc[$30][3] := '505030';
  sc[$31][0] := 'Q';       sc[$31][1] := 'Q';      sc[$31][2] := '49';    sc[$31][3] := '301450';
  sc[$32][0] := 'R';       sc[$32][1] := 'R';      sc[$32][2] := '50';    sc[$32][3] := '341050';
  sc[$33][0] := 'S';       sc[$33][1] := 'S';      sc[$33][2] := '51';    sc[$33][3] := '305014';
  sc[$34][0] := 'T';       sc[$34][1] := 'T';      sc[$34][2] := '52';    sc[$34][3] := '305410';
  sc[$35][0] := 'U';       sc[$35][1] := 'U';      sc[$35][2] := '53';    sc[$35][3] := '305050';
  sc[$36][0] := 'V';       sc[$36][1] := 'V';      sc[$36][2] := '54';    sc[$36][3] := '501034';
  sc[$37][0] := 'W';       sc[$37][1] := 'W';      sc[$37][2] := '55';    sc[$37][3] := '501430';
  sc[$38][0] := 'X';       sc[$38][1] := 'X';      sc[$38][2] := '56';    sc[$38][3] := '541030';
  sc[$39][0] := 'Y';       sc[$39][1] := 'Y';      sc[$39][2] := '57';    sc[$39][3] := '503014';
  sc[$3A][0] := 'Z';       sc[$3A][1] := 'Z';      sc[$3A][2] := '58';    sc[$3A][3] := '503410';
  sc[$3B][0] := '[';       sc[$3B][1] := '[';      sc[$3B][2] := '59';    sc[$3B][3] := '543010';
  sc[$3C][0] := '\';       sc[$3C][1] := '\';      sc[$3C][2] := '60';    sc[$3C][3] := '507010';
  sc[$3D][0] := ']';       sc[$3D][1] := ']';      sc[$3D][2] := '61';    sc[$3D][3] := '321610';
  sc[$3E][0] := '^';       sc[$3E][1] := '^';      sc[$3E][2] := '62';    sc[$3E][3] := '741010';
  sc[$3F][0] := '_';       sc[$3F][1] := '_';      sc[$3F][2] := '63';    sc[$3F][3] := '101236';
  sc[$40][0] := 'NUL';     sc[$40][1] := '`';      sc[$40][2] := '64';    sc[$40][3] := '101632';
  sc[$41][0] := 'SOH';     sc[$41][1] := 'a';      sc[$41][2] := '65';    sc[$41][3] := '121036';
  sc[$42][0] := 'STX';     sc[$42][1] := 'b';      sc[$42][2] := '66';    sc[$42][3] := '121630';
  sc[$43][0] := 'ETX';     sc[$43][1] := 'c';      sc[$43][2] := '67';    sc[$43][3] := '161032';
  sc[$44][0] := 'EOT';     sc[$44][1] := 'd';      sc[$44][2] := '68';    sc[$44][3] := '161230';
  sc[$45][0] := 'ENO';     sc[$45][1] := 'e';      sc[$45][2] := '69';    sc[$45][3] := '103216';
  sc[$46][0] := 'ACK';     sc[$46][1] := 'f';      sc[$46][2] := '70';    sc[$46][3] := '103612';
  sc[$47][0] := 'BEL';     sc[$47][1] := 'g';      sc[$47][2] := '71';    sc[$47][3] := '123016';
  sc[$48][0] := 'BS';      sc[$48][1] := 'h';      sc[$48][2] := '72';    sc[$48][3] := '123610';
  sc[$49][0] := 'HT';      sc[$49][1] := 'i';      sc[$49][2] := '73';    sc[$49][3] := '163012';
  sc[$4A][0] := 'LF';      sc[$4A][1] := 'j';      sc[$4A][2] := '74';    sc[$4A][3] := '163210';
  sc[$4B][0] := 'VT';      sc[$4B][1] := 'k';      sc[$4B][2] := '75';    sc[$4B][3] := '361210';
  sc[$4C][0] := 'FF';      sc[$4C][1] := 'l';      sc[$4C][2] := '76';    sc[$4C][3] := '321016';
  sc[$4D][0] := 'CR';      sc[$4D][1] := 'm';      sc[$4D][2] := '77';    sc[$4D][3] := '705010';
  sc[$4E][0] := 'SO';      sc[$4E][1] := 'n';      sc[$4E][2] := '78';    sc[$4E][3] := '361012';
  sc[$4F][0] := 'SI';      sc[$4F][1] := 'o';      sc[$4F][2] := '79';    sc[$4F][3] := '147010';
  sc[$50][0] := 'DLE';     sc[$50][1] := 'p';      sc[$50][2] := '80';    sc[$50][3] := '101272';
  sc[$51][0] := 'DC1';     sc[$51][1] := 'q';      sc[$51][2] := '81';    sc[$51][3] := '121072';
  sc[$52][0] := 'DC2';     sc[$52][1] := 'r';      sc[$52][2] := '82';    sc[$52][3] := '121270';
  sc[$53][0] := 'DC3';     sc[$53][1] := 's';      sc[$53][2] := '83';    sc[$53][3] := '107212';
  sc[$54][0] := 'DC4';     sc[$54][1] := 't';      sc[$54][2] := '84';    sc[$54][3] := '127012';
  sc[$55][0] := 'NAK';     sc[$55][1] := 'u';      sc[$55][2] := '85';    sc[$55][3] := '127210';
  sc[$56][0] := 'SYN';     sc[$56][1] := 'v';      sc[$56][2] := '86';    sc[$56][3] := '701212';
  sc[$57][0] := 'ETB';     sc[$57][1] := 'w';      sc[$57][2] := '87';    sc[$57][3] := '721012';
  sc[$58][0] := 'CAN';     sc[$58][1] := 'x';      sc[$58][2] := '88';    sc[$58][3] := '721210';
  sc[$59][0] := 'EM';      sc[$59][1] := 'y';      sc[$59][2] := '89';    sc[$59][3] := '303070';
  sc[$5A][0] := 'SUB';     sc[$5A][1] := 'z';      sc[$5A][2] := '90';    sc[$5A][3] := '307030';
  sc[$5B][0] := 'ESC';     sc[$5B][1] := '{';      sc[$5B][2] := '91';    sc[$5B][3] := '703030';
  sc[$5C][0] := 'FS';      sc[$5C][1] := '|';      sc[$5C][2] := '92';    sc[$5C][3] := '101074';
  sc[$5D][0] := 'GS';      sc[$5D][1] := '}';      sc[$5D][2] := '93';    sc[$5D][3] := '101470';
  sc[$5E][0] := 'RS';      sc[$5E][1] := '~';      sc[$5E][2] := '94';    sc[$5E][3] := '141070';
  sc[$5F][0] := 'US';      sc[$5F][1] := 'DEL';    sc[$5F][2] := '95';    sc[$5F][3] := '107014';
  sc[$60][0] := 'FNC3';    sc[$60][1] := 'FNC3';   sc[$60][2] := '96';    sc[$60][3] := '107410';
  sc[$61][0] := 'FNC2';    sc[$61][1] := 'FNC2';   sc[$61][2] := '97';    sc[$61][3] := '701014';
  sc[$62][0] := 'SHIFT';   sc[$62][1] := 'SHIFT';  sc[$62][2] := '98';    sc[$62][3] := '701410';
  sc[$63][0] := 'CODEC';   sc[$63][1] := 'CODEC';  sc[$63][2] := '99';    sc[$63][3] := '105070';
  sc[$64][0] := 'CODEB';   sc[$64][1] := 'FNC4';   sc[$64][2] := 'CODEB'; sc[$64][3] := '107050';
  sc[$65][0] := 'FNC4';    sc[$65][1] := 'CODEA';  sc[$65][2] := 'CODEA'; sc[$65][3] := '501070';
  sc[$66][0] := 'FNC1';    sc[$66][1] := 'FNC1';   sc[$66][2] := 'FNC1';  sc[$66][3] := '701050';

  se[0] := '301612';  // StartA
  se[1] := '301216';  // StartB
  se[2] := '301252';  // StartC
  se[3] := '3450103'; // Stop

  Dmy := FCode;

  if  Dmy = '' then
    begin
      Result := '';
      Exit;
    end;

  ShiftCode := -1;
  Code      := Ord(FStartChar) - Ord('A');
  Count     := 0;

  if (Code < 0) or (Code > 2) then
    begin
      Code := 0;
      FStartChar := Char('A');
    end;

  // スタートコード
  Module    := se[Code];
  SCC       := $67 + Code;

  // コード
  i := 1;
  while (i<=Length(Dmy)) do
    begin
      if Dmy[i] = '<' then
        begin
          Inc(i);
          if Dmy[i] ='<' then
            Command := Dmy[i]
          else
            begin
              Command := '';
              EndCmd := False;
              repeat
                Command := Command + Dmy[i];
                Inc(i);
                if (Dmy[i] = '>') or (Length(Dmy) < i) then
                  EndCmd := True;
              until(EndCmd);
              Command := UpperCase(Command);
            end
        end
      else if Dmy[i] = '>' then
        begin
          Inc(i);
          Command := Dmy[i]
        end
      else
        Command := Dmy[i];

      Index := SearchCode(Code,Command);
      if Index < 0 then
        begin
          Result := '';
          Exit;
        end
      else
        begin
          Inc(Count);
          SCC := SCC + (Index * Count);
          Module := Module + sc[Index][3];
          Inc(i);
          if Command = 'SHIFT' then
            begin
              ShiftCode := Code;
              // A->B,B->C,C->A
              Code := (Code + 1) mod 3;
            end
          else if (Command = 'CODEA') then
            Code := 0
          else if (Command = 'CODEB') then
            Code := 1
          else if (Command = 'CODEC') then
            Code := 2
          else
            begin
              if ShiftCode >= 0 then
                begin
                  Code := ShiftCode;
                  ShiftCode := -1;
                end;
            end;
        end;
    end;
  // チェックデジット
  SCC := (SCC mod 103);
  Module := Module + sc[SCC][3];
  // ストップコード
  Module := Module + se[3];

  FSymbol := FStartChar + Dmy;
  Command := sc[SCC][Code];
  if Command = '<' then
    FSymbol := FSymbol + '<<'
  else if Command = '>' then
    FSymbol := FSymbol + '>>'
  else if Length(Command) > 1 then
    FSymbol := FSymbol + '<' + Command + '>'
  else
    FSymbol := FSymbol + Command;

  FCDN := SCC;
  Result := Module;
end;

function TLRBarImage.Make_CODE93: String;
var
  i     ,
  j     ,
  len   : Integer;
  Chkc  ,
  Chkk  : LongInt;
  Dmy   ,
  pm    ,
  Module,
  tb    ,
  tc    : String;
  bp    : array [$00..$2E] of String;
  se    : array[0..1] of String;

begin
  tb := '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ-. $/+%!#&@';
  tc := '$%/+';
  bp[$00] := '141012'; // 0
  bp[$01] := '101214'; // 1
  bp[$02] := '101412'; // 2
  bp[$03] := '101610'; // 3
  bp[$04] := '121014'; // 4
  bp[$05] := '121212'; // 5
  bp[$06] := '121410'; // 6
  bp[$07] := '101016'; // 7
  bp[$08] := '141210'; // 8
  bp[$09] := '161010'; // 9
  bp[$0A] := '301014'; // A
  bp[$0B] := '301212'; // B
  bp[$0C] := '301410'; // C
  bp[$0D] := '321012'; // D
  bp[$0E] := '321210'; // E
  bp[$0F] := '341010'; // F
  bp[$10] := '103014'; // G
  bp[$11] := '103212'; // H
  bp[$12] := '103410'; // I
  bp[$13] := '123012'; // J
  bp[$14] := '143010'; // K
  bp[$15] := '101034'; // L
  bp[$16] := '101232'; // M
  bp[$17] := '101430'; // N
  bp[$18] := '121032'; // O
  bp[$19] := '141030'; // P
  bp[$1A] := '303012'; // Q
  bp[$1B] := '303210'; // R
  bp[$1C] := '301032'; // S
  bp[$1D] := '301230'; // T
  bp[$1E] := '321030'; // U
  bp[$1F] := '323010'; // V
  bp[$20] := '103032'; // W
  bp[$21] := '103230'; // X
  bp[$22] := '123030'; // Y
  bp[$23] := '125010'; // Z
  bp[$24] := '121050'; // -
  bp[$25] := '501012'; // .
  bp[$26] := '501210'; // [SPACE] | =
  bp[$27] := '521010'; // $
  bp[$28] := '103050'; // /
  bp[$29] := '105030'; // +
  bp[$2A] := '301050'; // %
  bp[$2B] := '121230'; // <$> | !
  bp[$2C] := '503010'; // <%> | #
  bp[$2D] := '501030'; // </> | &
  bp[$2E] := '123210'; // <+> | @

  { スタート・ストップキャラクタ }
  se[0] := '101070';   // [START] | (
  se[1] := '1010701';  // [STOP]  | )

  Dmy := FCode;

  if Dmy = '' then
  begin
    Result := '';
    Exit;
  end;

  { 制御コード部分の処理 }
  len := Length(Dmy);
  pm := '';
  i := 1;
  while (i <= len) do
  begin
    if Dmy[i] = '<' then
    begin
      if (i <= len - 2) and (Dmy[i + 2] = '>') then
      begin
        j := Pos(Dmy[i + 1], tc);
        if j = 0 then
          Break;
        pm := pm + tb[$2A + 1 + j];
        i := i + 2;
      end
      else
        Break;
    end
    else
    begin
      j := Pos(Dmy[i], tb);
      if (j = 0) or (j > $2A + 1) then
        Break;
      pm := pm + tb[j];
    end;
    inc(i);
  end;

  if i <= len then
  begin
    Result := '';
    Exit;
  end;

  { ２個のチェックデジットの付加(自動生成) }
  len := Length(pm);
  Chkc := 0;
  Chkk := 0;
  // すべての文字を与えられたコードに当てはめて加算
  for i:= len downto 1 do
    Chkc := Chkc + (len - i + 1) * (Pos(pm[i], tb) - 1);
  // モジュラス４７でチェックデジットを算出
  Chkc := Chkc mod 47;

  // すべての文字を与えられたコードに当てはめて加算
  Chkk := Chkk + Chkc;
  for i:= len downto 1 do
    Chkk := Chkk + (len - i + 2) * (Pos(pm[i], tb) - 1);
  // モジュラス４７でチェックデジットを算出
  Chkk := Chkk mod 47;

  {バーコードの作成}
  Module := '';
  Module := Module + se[0];               // スタートキャラクタ'('を追加
  for i:=1 to len do
    Module := Module + bp[Pos(pm[i], tb) - 1];
  Module := Module + bp[Chkc] + bp[Chkk]; // チェックデジットを追加
  Module := Module + se[1];               // ストップキャラクタ')'を追加

  FSymbol := '(' + Dmy;
  if Chkc > $2A then
    FSymbol := FSymbol + '<' + tc[Chkc - $2A] + '>'
  else
    FSymbol := FSymbol + tb[Chkc + 1];
  if Chkk > $2A then
    FSymbol := FSymbol + '<' + tc[Chkk - $2A] + '>'
  else
    FSymbol := FSymbol + tb[Chkk + 1];
  FSymbol := FSymbol + ')';

  FCDN := Chkc + Chkk * 100; // ２個のチェックデジットを４桁の１０進数で表現
  Result := Module;
end;

procedure TLRBarImage.PaintBarcode;
var
  Bmp: TBitmap;
  Jpg: TJPEGImage;
begin
  if FBarEnabled = False then  // 一時的にバーコード表示を停止している状態
    Exit;

  { Picture が、JPEG 形式のデータを持っている場合は、いったん }
  { それを Bitmap 形式のデータに変換しておきます。}
  if Picture.Graphic is TJPEGImage then
  begin
    Bmp := TBitmap.Create;
    try
      Bmp.Assign(Picture.Graphic);
      Picture.Assign(Bmp);
    finally
      Bmp.Free;
    end;
  end;

  { Picture が、どの様な形式のデータを持っている場合でも、いったん }
  { Picture の Bitmap を有効な状態にしておきます。従って、もともとの }
  { データが、Bitmap と JPEG 以外の場合は、この時点で、保持していた }
  { データが破棄されることになります。}
  Picture.Bitmap.PixelFormat := pf32bit; // どの様な環境でもこの値に固定します。
  if Picture.Bitmap.Empty = True then
  begin
    Picture.Bitmap.Width := Width;
    Picture.Bitmap.Height := Height;
  end;

  if FCode = '' then
    Picture := nil
  else
    begin
      if FBarPicture in [picBMP, picJPEG]  then
        PaintBarcodeB        // Picture.Bitmap への描画を行なう場合
      else
        PaintBarcodeM;       // Picture.Metafile への描画を行なう場合
    end;

  { JPEG 表示の場合は、ここで Picture の Bitmap データを }
  { JPEG 形式のデータに変換しておきます。}
  if FBarPicture = picJPEG then
  begin
    Jpg := TJPEGImage.Create;
    try
      Jpg.Assign(Picture.Graphic);
      Picture.Assign(Jpg);
    finally
      Jpg.Free;
    end;
  end;

  Invalidate;
end;

procedure TLRBarImage.PaintBarcodeB;
var
  Position, Position2, Cnt, i, w: Integer;
  LineWidth, LineHeight: Integer;       // バーコードのバー部分の大きさ
  SymbolWidth, SymbolHeight: Integer;   // バーコード文字列の部分の大きさ
  CmtUpWidth, CmtUpHeight: Integer;     // 上部のコメント部分の大きさ
  CmtDownWidth, CmtDownHeight: Integer; // 下部のコメント部分の大きさ
  PositionUp, PositionDown: Integer;    // 各コメントの表示開始位置
  ww, hh: Integer;
  bl, bt: Integer;
  asz, srh: Boolean;
  Module: String;
  OrgFont: TFont;

begin
  FBarDisped := False;
  FSymbol := '';
  FCDN := -1; // チェックデジットが存在しない場合の値

  if FCode = '' then
    begin
      Picture.Bitmap.Canvas.Brush.Color := FBackColor;
      Picture.Bitmap.Canvas.FillRect(Rect(0, 0, Picture.Bitmap.Width, Picture.Bitmap.Height));
      Exit;
    end;

  bl := FBarLeft;
  bt := FBarTop;

  asz := AutoSize;
  srh := Stretch;
  AutoSize := False;
  Stretch := False;
  Module := '';

  case FBarType of
    cbarJAN: Module := Make_JAN;
    cbarUPC: Module := Make_UPC;
    cbarCODE39: Module := Make_CODE39;
    cbarNW7: Module := Make_NW7;
    cbarITF: Module := Make_ITF;
    cbarCTF: Module := Make_CTF;
    cbarIATA: Module := Make_IATA;
    cbarMATRIX: Module := Make_MATRIX;
    cbarNEC: Module := Make_NEC;
    cbarCUSTOMER: Module := Make_CUSTOMER;
    cbarCODE128: Module := Make_CODE128;
    cbarCODE93: Module := Make_CODE93;
  end;

  if Module = '' then
  begin
    Picture.Bitmap.Canvas.Brush.Color := FBackColor;
    Picture.Bitmap.Canvas.FillRect(Rect(0, 0, Picture.Bitmap.Width, Picture.Bitmap.Height));
    AutoSize := asz;
    Stretch := srh;
    Exit;
  end;

  { デバッグモードの場合 }
  if FBarDebug = True then
  begin
    FBarDisped := True;
    AutoSize := asz;
    Stretch := srh;
    Exit;
  end;

  { まず、バーコード文字列の部分の大きさを求める。}
  if FSymbolDisp = True then
  begin
    OrgFont := Picture.Bitmap.Canvas.Font;
    Picture.Bitmap.Canvas.Font := FBarFont;
    SymbolWidth := Picture.Bitmap.Canvas.TextWidth(FSymbol);
    SymbolHeight := Picture.Bitmap.Canvas.TextHeight(FSymbol);
    Picture.Bitmap.Canvas.Font := OrgFont;
  end
  else
  begin
    SymbolWidth := 0;
    SymbolHeight := 0;
  end;

  { 各コメント部分の大きさを求める。}
  OrgFont := Picture.Bitmap.Canvas.Font;
  Picture.Bitmap.Canvas.Font := FBarFont;
  if FCommentUp <> '' then
  begin
    CmtUpWidth := Picture.Bitmap.Canvas.TextWidth(FCommentUp);
    CmtUpHeight := Picture.Bitmap.Canvas.TextHeight(FCommentUp);
  end
  else
  begin
    CmtUpWidth := 0;
    CmtUpHeight := 0;
  end;

  if FCommentDown <> '' then
  begin
    CmtDownWidth := Picture.Bitmap.Canvas.TextWidth(FCommentDown);
    CmtDownHeight := Picture.Bitmap.Canvas.TextHeight(FCommentDown);
  end
  else
  begin
    CmtDownWidth := 0;
    CmtDownHeight := 0;
  end;
  Picture.Bitmap.Canvas.Font := OrgFont;

  { バーコードのバー部分の大きさを求める。}
  if FBarType = cbarCUSTOMER then
  begin
    LineWidth := WidthCUSTOMER(Module);
    LineHeight := 3 * FScale * 2;
  end
  else
  begin
    LineWidth := 0;
    for i := 1 to Length(Module) do
    begin
      w := ((StrToInt(Module[i]) div 2) + 1) * FScale;
      if (FBarType >= cbarCODE39) and (FBarType <= cbarNEC) then
      begin
        if w > FScale then  // ワイドバーあるいは、ワイドスペースの場合
          w := (w * FElementRatio) div 20;
      end;
      LineWidth := LineWidth + w;
    end;
    if FBarHeight < (SymbolHeight + 6) then
      FBarHeight := SymbolHeight + 6;
    LineHeight := FBarHeight - SymbolHeight;
  end;

  if LineWidth = 0 then
  begin
    AutoSize := asz;
    Stretch := srh;
    Exit;
  end;

  { バーコードの基本の大きさを求める。}
  if LineWidth > SymbolWidth then
    FBarWidth := LineWidth
  else
    FBarWidth := SymbolWidth;

  if FBarType = cbarCUSTOMER then
    FBarHeight := LineHeight + SymbolHeight;

  ww := FBarSpaceLeft + FBarWidth + FBarSpaceRight;
  hh := FBarSpaceUp + FBarHeight + FBarSpaceDown;

  if FMatch = False then
  begin
    if FAngle = 90 then
    begin
      FBarLeft := bt;
      FBarTop := Width - 1 - bl - hh + 1;
    end
    else if FAngle = 180 then
    begin
      FBarLeft := Width - 1 - bl - ww + 1;
      FBarTop := Height - 1 - bt - hh + 1;
    end
    else if FAngle = 270 then
    begin
      FBarLeft := Height - 1 - bt - ww + 1;
      FBarTop := bl;
    end;
    RotateBitmap(360 - FAngle);  // -FAngle 度分回転する。
  end;

  ww := FBarLeft + ww;
  hh := FBarTop + hh;

  { コントロールのサイズをバーコードのサイズに合わせる場合 }
  if FMatch = True then
  begin
    Width := ww;
    Height := hh;
    Picture.Bitmap.Width := Width;
    Picture.Bitmap.Height := Height;
  end;

  { Canvas の Pen と Brush を設定する。 }
  Picture.Bitmap.Canvas.Pen.Color := FBarColor;
  Picture.Bitmap.Canvas.Pen.Style := psSolid;
  Picture.Bitmap.Canvas.Brush.Color := FBackColor;

  { バーコードの背景を描く。 }
  if FTransparent = False then
  begin
    Picture.Bitmap.Canvas.Brush.Style := bsSolid;
    if FMatch = True then
      Picture.Bitmap.Canvas.FillRect(Rect(0, 0, ww, hh))
    else
      Picture.Bitmap.Canvas.FillRect(Rect(FBarLeft, FBarTop, ww, hh));
  end;
  Picture.Bitmap.Canvas.Brush.Style := bsClear;

  { バーコードのバー部分を描く。}
  if LineWidth > SymbolWidth then
  begin
    Position := 0;
    Position2 := (LineWidth - SymbolWidth) div 2;
  end
  else
  begin
    Position := (SymbolWidth - LineWidth) div 2;
    Position2 := 0;
  end;

  if FBarType = cbarCUSTOMER then
  begin
    Picture.Bitmap.Canvas.Brush.Color := FBarColor;
    Picture.Bitmap.Canvas.Brush.Style := bsSolid;

    for Cnt := 0 to Length(Module) - 1 do
      DrawCUSTOMER(Position, Cnt, FScale, StrToInt(Module[Cnt + 1]));

    Picture.Bitmap.Canvas.Brush.Color := FBackColor;
    Picture.Bitmap.Canvas.Brush.Style := bsClear;
  end
  else
  begin
    for Cnt := 1 to Length(Module) do
    begin
      LineWidth := ((StrToInt(Module[Cnt]) div 2) + 1) * FScale;
      if (FBarType >= cbarCODE39) and (FBarType <= cbarNEC) then
      begin
        if LineWidth > FScale then  // ワイドバーあるいは、ワイドスペースの場合
          LineWidth := (LineWidth * FElementRatio) div 20;
      end;
      if (StrToInt(Module[Cnt]) mod 2) = 0 then
      // 空白
        Position := Position + LineWidth
      else
      // バー
      begin
        for i := 1 to LineWidth do
        begin
          Picture.Bitmap.Canvas.MoveTo(FBarLeft + FBarSpaceLeft + Position,
           FBarTop + FBarSpaceUp);
          Picture.Bitmap.Canvas.LineTo(FBarLeft + FBarSpaceLeft + Position,
           FBarTop + FBarSpaceUp + LineHeight);
          Inc(Position);
        end;
      end;
    end;
  end;

  { バーコード文字列の部分を描く。}
  if (FSymbolDisp = True) and (FSymbol <> '') then
  begin
    OrgFont := Picture.Bitmap.Canvas.Font;
    Picture.Bitmap.Canvas.Font := FBarFont;
    Picture.Bitmap.Canvas.TextOut(FBarLeft + FBarSpaceLeft + Position2,
     FBarTop + FBarSpaceUp + LineHeight, FSymbol);
    Picture.Bitmap.Canvas.Font := OrgFont;
  end;

  { バーコードの周囲にフレーム(外枠)を描く場合の処理 }
  if FBarFrame > 0 then
  begin
    if FBarType = cbarITF then  // ITF のフレーム(外枠)
      DrawFrame(Canvas,
           Rect(FBarLeft,
                FBarTop + FBarSpaceUp,
                FBarLeft + FBarSpaceLeft + FBarWidth + FBarSpaceRight,
                FBarTop + FBarSpaceUp + LineHeight))
    else                        // ITF 以外のフレーム(外枠)
      DrawFrame(Canvas,
           Rect(FBarLeft,
                FBarTop + CmtUpHeight,
                FBarLeft + FBarSpaceLeft + FBarWidth + FBarSpaceRight,
                FBarTop + FBarSpaceUp + FBarHeight + FBarSpaceDown
                 - CmtDownHeight));
  end;

  { バーコードの上下にコメントを描く場合の処理 }
  OrgFont := Picture.Bitmap.Canvas.Font;
  Picture.Bitmap.Canvas.Font := FBarFont;
  if FCommentUp <> '' then
  begin
    case FCommentUpLoc of
      locLeft:
        PositionUp := 0;
      locCenter:
        PositionUp := (FBarSpaceLeft + FBarWidth + FBarSpaceRight
         - CmtUpWidth) div 2;
      locRight:
        PositionUp := FBarSpaceLeft + FBarWidth + FBarSpaceRight - CmtUpWidth;
    else
      PositionUp := 0;
    end;
    if PositionUp < 0 then
      PositionUp := 0;
    Picture.Bitmap.Canvas.TextOut(FBarLeft + PositionUp, FBarTop, FCommentUp);
  end;

  if FCommentDown <> '' then
  begin
    case FCommentDownLoc of
      locLeft:
        PositionDown := 0;
      locCenter:
        PositionDown := (FBarSpaceLeft + FBarWidth + FBarSpaceRight
         - CmtDownWidth) div 2;
      locRight:
        PositionDown := FBarSpaceLeft + FBarWidth + FBarSpaceRight
         - CmtDownWidth;
    else
      PositionDown := 0;
    end;
    if PositionDown < 0 then
      PositionDown := 0;
    Picture.Bitmap.Canvas.TextOut(FBarLeft + PositionDown,
     FBarTop + FBarSpaceUp + FBarHeight + FBarSpaceDown - CmtDownHeight,
      FCommentDown);
  end;
  Picture.Bitmap.Canvas.Font := OrgFont;

  RotateBitmap(FAngle);  // FAngle 度分回転する。

  AutoSize := asz;
  Stretch := srh;

  if FMatch = False then
  begin
    FBarLeft := bl;
    FBarTop := bt;
  end;
  FBarDisped := True;
end;

procedure TLRBarImage.PaintBarcodeM;
var
  Position, Position2, Cnt, i, w: Integer;
  LineWidth, LineHeight: Integer;       // バーコードのバー部分の大きさ
  SymbolWidth, SymbolHeight: Integer;   // バーコード文字列の部分の大きさ
  CmtUpWidth, CmtUpHeight: Integer;     // 上部のコメント部分の大きさ
  CmtDownWidth, CmtDownHeight: Integer; // 下部のコメント部分の大きさ
  PositionUp, PositionDown: Integer;    // 各コメントの表示開始位置
  asz, srh: Boolean;
  Module: String;
  OrgFont: TFont;

begin
  FBarDisped := False;
  FSymbol := '';
  FCDN := -1; // チェックデジットが存在しない場合の値

  if FCode = '' then
    begin
      FMfCanvas := TMetafileCanvas.Create(Picture.Metafile, GetDC(0));
      FMfCanvas.Brush.Color := FBackColor;
      FMfCanvas.FillRect(Rect(0, 0, Width, Height));
      FMfCanvas.Free;
      Exit;
    end;

  asz := AutoSize;
  srh := Stretch;
  AutoSize := False;
  Stretch := False;
  Module := '';

  case FBarType of
    cbarJAN: Module := Make_JAN;
    cbarUPC: Module := Make_UPC;
    cbarCODE39: Module := Make_CODE39;
    cbarNW7: Module := Make_NW7;
    cbarITF: Module := Make_ITF;
    cbarCTF: Module := Make_CTF;
    cbarIATA: Module := Make_IATA;
    cbarMATRIX: Module := Make_MATRIX;
    cbarNEC: Module := Make_NEC;
    cbarCUSTOMER: Module := Make_CUSTOMER;
    cbarCODE128: Module := Make_CODE128;
    cbarCODE93: Module := Make_CODE93;
  end;

  if Module = '' then
    begin
      FMfCanvas := TMetafileCanvas.Create(Picture.Metafile, GetDC(0));
      FMfCanvas.Brush.Color := FBackColor;
      FMfCanvas.FillRect(Rect(0, 0, Width, Height));
      FMfCanvas.Free;
      AutoSize := asz;
      Stretch := srh;
      Exit;
    end;

  { デバッグモードの場合 }
  if FBarDebug = True then
  begin
    FBarDisped := True;
    AutoSize := asz;
    Stretch := srh;
    Exit;
  end;

  { まず、バーコード文字列の部分の大きさを求める。}
  if FSymbolDisp = True then
  begin
    OrgFont := Picture.Bitmap.Canvas.Font;
    Picture.Bitmap.Canvas.Font := FBarFont;
    SymbolWidth := Picture.Bitmap.Canvas.TextWidth(FSymbol);
    SymbolHeight := Picture.Bitmap.Canvas.TextHeight(FSymbol);
    Picture.Bitmap.Canvas.Font := OrgFont;
  end
  else
  begin
    SymbolWidth := 0;
    SymbolHeight := 0;
  end;

  { 各コメント部分の大きさを求める。}
  OrgFont := Picture.Bitmap.Canvas.Font;
  Picture.Bitmap.Canvas.Font := FBarFont;
  if FCommentUp <> '' then
  begin
    CmtUpWidth := Picture.Bitmap.Canvas.TextWidth(FCommentUp);
    CmtUpHeight := Picture.Bitmap.Canvas.TextHeight(FCommentUp);
  end
  else
  begin
    CmtUpWidth := 0;
    CmtUpHeight := 0;
  end;

  if FCommentDown <> '' then
  begin
    CmtDownWidth := Picture.Bitmap.Canvas.TextWidth(FCommentDown);
    CmtDownHeight := Picture.Bitmap.Canvas.TextHeight(FCommentDown);
  end
  else
  begin
    CmtDownWidth := 0;
    CmtDownHeight := 0;
  end;
  Picture.Bitmap.Canvas.Font := OrgFont;

  { バーコードのバー部分の大きさを求める。}
  if FBarType = cbarCUSTOMER then
  begin
    LineWidth := WidthCUSTOMER(Module);
    LineHeight := 3 * FScale * 2;
  end
  else
  begin
    LineWidth := 0;
    for i := 1 to Length(Module) do
    begin
      w := ((StrToInt(Module[i]) div 2) + 1) * FScale;
      if (FBarType >= cbarCODE39) and (FBarType <= cbarNEC) then
      begin
        if w > FScale then  // ワイドバーあるいは、ワイドスペースの場合
          w := (w * FElementRatio) div 20;
      end;
      LineWidth := LineWidth + w;
    end;
    if FBarHeight < (SymbolHeight + 6) then
      FBarHeight := SymbolHeight + 6;
    LineHeight := FBarHeight - SymbolHeight;
  end;

  if LineWidth = 0 then
  begin
    AutoSize := asz;
    Stretch := srh;
    Exit;
  end;

  { バーコードの基本の大きさを求める。}
  if LineWidth > SymbolWidth then
    FBarWidth := LineWidth
  else
    FBarWidth := SymbolWidth;

  if FBarType = cbarCUSTOMER then
    FBarHeight := LineHeight + SymbolHeight;

  { コントロールのサイズをバーコードのサイズに合わせる。}
  Width := FBarLeft + FBarSpaceLeft + FBarWidth + FBarSpaceRight;
  Height := FBarTop + FBarSpaceUp + FBarHeight + FBarSpaceDown;
  Picture.Metafile.Width := Width;
  Picture.Metafile.Height := Height;

  { メタファイルのキャンバスを用意する。 }
  FMfCanvas := TMetafileCanvas.Create(Picture.Metafile, GetDC(0));

  { FMfCanvas の Pen と Brush を設定する。 }
  FMfCanvas.Pen.Color := FBarColor;
  FMfCanvas.Pen.Style := psSolid;
  FMfCanvas.Brush.Color := FBackColor;

  { バーコードの背景を描く。 }
  FMfCanvas.Brush.Style := bsSolid;
  FMfCanvas.FillRect(Rect(0, 0, Width, Height));
  FMfCanvas.Brush.Style := bsClear;

  { バーコードのバー部分を描く。}
  if LineWidth > SymbolWidth then
  begin
    Position := 0;
    Position2 := (LineWidth - SymbolWidth) div 2;
  end
  else
  begin
    Position := (SymbolWidth - LineWidth) div 2;
    Position2 := 0;
  end;

  if FBarType = cbarCUSTOMER then
  begin
    FMfCanvas.Brush.Color := FBarColor;
    FMfCanvas.Brush.Style := bsSolid;

    for Cnt := 0 to Length(Module) - 1 do
      DrawCUSTOMER(Position, Cnt, FScale, StrToInt(Module[Cnt + 1]));

    FMfCanvas.Brush.Color := FBackColor;
    FMfCanvas.Brush.Style := bsClear;
  end
  else
  begin
    for Cnt := 1 to Length(Module) do
    begin
      LineWidth := ((StrToInt(Module[Cnt]) div 2) + 1) * FScale;
      if (FBarType >= cbarCODE39) and (FBarType <= cbarNEC) then
      begin
        if LineWidth > FScale then  // ワイドバーあるいは、ワイドスペースの場合
          LineWidth := (LineWidth * FElementRatio) div 20;
      end;
      if (StrToInt(Module[Cnt]) mod 2) = 0 then
      // 空白
        Position := Position + LineWidth
      else
      // バー
      begin
        for i := 1 to LineWidth do
        begin
          FMfCanvas.MoveTo(FBarLeft + FBarSpaceLeft + Position,
           FBarTop + FBarSpaceUp);
          FMfCanvas.LineTo(FBarLeft + FBarSpaceLeft + Position,
           FBarTop + FBarSpaceUp + LineHeight);
          Inc(Position);
        end;
      end;
    end;
  end;

  { バーコード文字列の部分を描く。}
  if (FSymbolDisp = True) and (FSymbol <> '') then
  begin
    FMfCanvas.Font := FBarFont;
    FMfCanvas.TextOut(FBarLeft + FBarSpaceLeft + Position2,
     FBarTop + FBarSpaceUp + LineHeight, FSymbol);
  end;

  { バーコードの周囲にフレーム(外枠)を描く場合の処理 }
  if FBarFrame > 0 then
  begin
    if FBarType = cbarITF then  // ITF のフレーム(外枠)
      DrawFrame(FMfCanvas,
           Rect(FBarLeft,
                FBarTop + FBarSpaceUp,
                FBarLeft + FBarSpaceLeft + FBarWidth + FBarSpaceRight,
                FBarTop + FBarSpaceUp + LineHeight))
    else                        // ITF 以外のフレーム(外枠)
      DrawFrame(FMfCanvas,
           Rect(FBarLeft,
                FBarTop + CmtUpHeight,
                FBarLeft + FBarSpaceLeft + FBarWidth + FBarSpaceRight,
                FBarTop + FBarSpaceUp + FBarHeight + FBarSpaceDown
                 - CmtDownHeight));
  end;

  { バーコードの上下にコメントを描く場合の処理 }
  FMfCanvas.Font := FBarFont;
  if FCommentUp <> '' then
  begin
    case FCommentUpLoc of
      locLeft:
        PositionUp := 0;
      locCenter:
        PositionUp := (FBarSpaceLeft + FBarWidth + FBarSpaceRight
         - CmtUpWidth) div 2;
      locRight:
        PositionUp := FBarSpaceLeft + FBarWidth + FBarSpaceRight - CmtUpWidth;
    else
      PositionUp := 0;
    end;
    if PositionUp < 0 then
      PositionUp := 0;
    FMfCanvas.TextOut(FBarLeft + PositionUp, FBarTop, FCommentUp);
  end;

  if FCommentDown <> '' then
  begin
    case FCommentDownLoc of
      locLeft:
        PositionDown := 0;
      locCenter:
        PositionDown := (FBarSpaceLeft + FBarWidth + FBarSpaceRight
         - CmtDownWidth) div 2;
      locRight:
        PositionDown := FBarSpaceLeft + FBarWidth + FBarSpaceRight
         - CmtDownWidth;
    else
      PositionDown := 0;
    end;
    if PositionDown < 0 then
      PositionDown := 0;
    FMfCanvas.TextOut(FBarLeft + PositionDown,
     FBarTop + FBarSpaceUp + FBarHeight + FBarSpaceDown - CmtDownHeight,
      FCommentDown);
  end;

  FMfCanvas.Free;        // 開放して初めて描画される。

  { なぜか１ピクセル分つぶれる(どこかの線がおかしくなる)ので表示後に }
  { 元にもどす。これは、M&I さんのご指摘により修正した部分です。}
  // 2001/08/05 Modify M&I
  Picture.Metafile.Width := Picture.Metafile.Width + 1;
  Picture.Metafile.Height := Picture.Metafile.Height + 1;

  AutoSize := asz;
  Stretch := srh;
  FBarDisped := True;
end;

procedure TLRBarImage.ReverseBitmap;
type
  TTriple = packed record
    B, G, R: Byte;
  end;

  TTripleArray = array[0..40000000] of TTriple;
  PTripleArray = ^TTripleArray;

  TDWordArray = array[0..100000000] of DWORD;
  PDWordArray = ^TDWordArray;
var
  NewBitmap: TBitmap;
  x, y, i: Integer;
  w, h: Integer;
  BitCount: Integer;
  DS: TDIBSection;
  OldBitmap: TBitmap;
  Bits: Byte;
  Index: Integer;
  SourceScanline, DestScanline: array of Pointer;
begin
  if (Picture.Graphic = Nil) or (not (FBarPicture in [picBMP, picJPEG])) or
   (FReverse = False) then
    Exit;

  Picture.Bitmap.HandleType := bmDIB;

  OldBitmap := (Picture.Graphic as TBitmap);

  NewBitmap := TBitmap.Create;
  try
    w := OldBitmap.Width;
    h := OldBitmap.Height;

    NewBitmap.PixelFormat := OldBitmap.PixelFormat;
    NewBitmap.Width := w;
    NewBitmap.Height := h;
    NewBitmap.Palette := CopyPalette(OldBitmap.Palette);

    GetObject(NewBitmap.Handle, SizeOf(TDIBSection), @DS);
    BitCount := DS.dsBmih.biBitCount;
    SetLength(SourceScanline, h);
    SetLength(DestScanline, h);
    for i := 0 to h - 1 do
      SourceScanline[i] := OldBitmap.Scanline[i];

    for i := 0 to h - 1 do
      DestScanline[i] := NewBitmap.Scanline[i];

    for y := 0 to h - 1 do
    begin
      for x := 0 to w - 1 do
      begin
        case Bitcount of
          32: PDWordArray(DestScanline[y])^[x] :=
                PDWordArray(SourceScanline[y])^[w - 1 - x];
          24: PTripleArray(DestScanline[y])^[x] :=
                PTripleArray(SourceScanline[y])^[w - 1 - x];
          16: PWordArray(DestScanline[y])^[x] :=
                PWordArray(SourceScanline[y])^[w - 1 - x];
          8: PByteArray(DestScanline[y])^[x] :=
                PByteArray(SourceScanline[y])^[w - 1 - x];
          4:
          begin
            Index := w - 1 - x;
            Bits := PByteArray(SourceScanline[y])^[Index div 2];
            Bits := (Bits shr (4 * (1 - Index mod 2))) and $0f;
            PByteArray(DestScanline[y])^[x div 2] :=
             (PByteArray(DestScanline[y])^[x div 2] and
             not ($f0 shr (4 * (x mod 2)))) or
             (Bits shl (4 * (1 - x mod 2)));
          end;
          1:
          begin
            Index := w - 1 - x;
            Bits := PByteArray(SourceScanline[y])^[Index div 8];
            Bits := (Bits shr (7 - Index mod 8)) and $01;
            PByteArray(DestScanline[y])^[x div 8] :=
             (PByteArray(DestScanline[y])^[x div 8] and
             not ($80 shr (x mod 8))) or
             (Bits shl (7 - x mod 8));
          end;
        end;
      end;
    end;
    Picture.Graphic := NewBitmap;
  finally
    NewBitmap.Free;
  end;
end;

procedure TLRBarImage.RotateBitmap(Degree: Integer);
type
  TTriple = packed record
    B, G, R: Byte;
  end;

  TTripleArray = array[0..40000000] of TTriple;
  PTripleArray = ^TTripleArray;

  TDWordArray = array[0..100000000] of DWORD;
  PDWordArray = ^TDWordArray;
var
  NewBitmap: TBitmap;
  x, y, i: Integer;
  w, h: Integer;
  ww, hh: Integer;
  asz, srh: Boolean;
  BitCount: Integer;
  DS: TDIBSection;
  OldBitmap: TBitmap;
  Bits: Byte;
  Index: Integer;
  SourceScanline, DestScanline: array of Pointer;
begin
  if (Picture.Graphic = Nil) or (not (FBarPicture in [picBMP, picJPEG])) then
    Exit;

  if ((Degree <> 90) and (Degree <> 180) and (Degree <> 270)) then
  begin
    ReverseBitmap;
    Exit;
  end;

  asz := AutoSize;
  srh := Stretch;
  AutoSize := False;
  Stretch := False;

  ww := Width;
  hh := Height;

  Picture.Bitmap.HandleType := bmDIB;

  OldBitmap := (Picture.Graphic as TBitmap);

  NewBitmap := TBitmap.Create;
  try
    if Degree = 180 then
    begin
      w := OldBitmap.Width;
      h := OldBitmap.Height;
    end
    else
    begin
      w := OldBitmap.Height;
      h := OldBitmap.Width;
    end;

    NewBitmap.PixelFormat := OldBitmap.PixelFormat;
    NewBitmap.Width := w;
    NewBitmap.Height := h;
    NewBitmap.Palette := CopyPalette(OldBitmap.Palette);

    if Degree = 90 then
    begin
      GetObject(NewBitmap.Handle, SizeOf(TDIBSection), @DS);
      BitCount := DS.dsBmih.biBitCount;
      SetLength(SourceScanline, w);
      SetLength(DestScanline, h);
      for i := 0 to w - 1 do
        SourceScanline[i] := OldBitmap.Scanline[i];

      for i := 0 to h - 1 do
        DestScanline[i] := NewBitmap.Scanline[i];

      for y := 0 to h - 1 do
      begin
        for x := 0 to w - 1 do
        begin
          case Bitcount of
            32: PDWordArray(DestScanline[y])^[x] :=
                  PDWordArray(SourceScanline[w - 1 - x])^[y];
            24: PTripleArray(DestScanline[y])^[x] :=
                  PTripleArray(SourceScanline[w - 1 - x])^[y];
            16: PWordArray(DestScanline[y])^[x] :=
                  PWordArray(SourceScanline[w - 1 - x])^[y];
            8: PByteArray(DestScanline[y])^[x] :=
                  PByteArray(SourceScanline[w - 1 - x])^[y];
            4:
            begin
              Index := y;
              Bits := PByteArray(SourceScanline[w - 1 - x])^[Index div 2];
              Bits := (Bits shr (4 * (1 - Index mod 2))) and $0f;
              PByteArray(DestScanline[y])^[x div 2] :=
               (PByteArray(DestScanline[y])^[x div 2] and
               not ($f0 shr (4 * (x mod 2)))) or
               (Bits shl (4 * (1 - x mod 2)));
            end;
            1:
            begin
              Index := y;
              Bits := PByteArray(SourceScanline[w - 1 - x])^[Index div 8];
              Bits := (Bits shr (7 - Index mod 8)) and $01;
              PByteArray(DestScanline[y])^[x div 8] :=
               (PByteArray(DestScanline[y])^[x div 8] and
               not ($80 shr (x mod 8))) or
               (Bits shl (7 - x mod 8));
            end;
          end;
        end;
      end;
      Width := hh;
      Height := ww;
    end
    else if Degree = 180 then
    begin
      GetObject(NewBitmap.Handle, SizeOf(TDIBSection), @DS);
      BitCount := DS.dsBmih.biBitCount;
      SetLength(SourceScanline, h);
      SetLength(DestScanline, h);
      for i := 0 to h - 1 do
        SourceScanline[i] := OldBitmap.Scanline[i];

      for i := 0 to h - 1 do
        DestScanline[i] := NewBitmap.Scanline[i];

      for y := 0 to h - 1 do
      begin
        for x := 0 to w - 1 do
        begin
          case Bitcount of
            32: PDWordArray(DestScanline[y])^[x] :=
                  PDWordArray(SourceScanline[h - 1 - y])^[w - 1 - x];
            24: PTripleArray(DestScanline[y])^[x] :=
                  PTripleArray(SourceScanline[h - 1 - y])^[w - 1 - x];
            16: PWordArray(DestScanline[y])^[x] :=
                  PWordArray(SourceScanline[h - 1 - y])^[w - 1 - x];
            8: PByteArray(DestScanline[y])^[x] :=
                  PByteArray(SourceScanline[h - 1 - y])^[w - 1 - x];
            4:
            begin
              Index := w - 1 - x;
              Bits := PByteArray(SourceScanline[h - 1 - y])^[Index div 2];
              Bits := (Bits shr (4 * (1 - Index mod 2))) and $0f;
              PByteArray(DestScanline[y])^[x div 2] :=
               (PByteArray(DestScanline[y])^[x div 2] and
               not ($f0 shr (4 * (x mod 2)))) or
               (Bits shl (4 * (1 - x mod 2)));
            end;
            1:
            begin
              Index := w - 1 - x;
              Bits := PByteArray(SourceScanline[h - 1 - y])^[Index div 8];
              Bits := (Bits shr (7 - Index mod 8)) and $01;
              PByteArray(DestScanline[y])^[x div 8] :=
               (PByteArray(DestScanline[y])^[x div 8] and
               not ($80 shr (x mod 8))) or
               (Bits shl (7 - x mod 8));
            end;
          end;
        end;
      end;
    end
    else if Degree = 270 then
    begin
      GetObject(NewBitmap.Handle, SizeOf(TDIBSection), @DS);
      BitCount := DS.dsBmih.biBitCount;
      SetLength(SourceScanline, w);
      SetLength(DestScanline, h);
      for i := 0 to w - 1 do
        SourceScanline[i] := OldBitmap.Scanline[i];

      for i := 0 to h - 1 do
        DestScanline[i] := NewBitmap.Scanline[i];

      for y := 0 to h - 1 do
      begin
        for x := 0 to w - 1 do
        begin
          case Bitcount of
            32: PDWordArray(DestScanline[y])^[x] :=
                  PDWordArray(SourceScanline[x])^[h - 1 - y];
            24: PTripleArray(DestScanline[y])^[x] :=
                  PTripleArray(SourceScanline[x])^[h - 1 - y];
            16: PWordArray(DestScanline[y])^[x] :=
                  PWordArray(SourceScanline[x])^[h - 1 - y];
            8: PByteArray(DestScanline[y])^[x] :=
                  PByteArray(SourceScanline[x])^[h - 1 - y];
            4:
            begin
              Index := h - 1 - y;
              Bits := PByteArray(SourceScanline[x])^[Index div 2];
              Bits := (Bits shr (4*(1 - Index mod 2))) and $0f;
              PByteArray(DestScanline[y])^[x div 2] :=
               (PByteArray(DestScanline[y])^[x div 2] and
               not ($f0 shr (4*(x mod 2)))) or
               (Bits shl (4*(1 - x mod 2)));
            end;
            1:
            begin
              Index := h - 1 - y;
              Bits := PByteArray(SourceScanline[x])^[Index div 8];
              Bits := (Bits shr (7 - Index mod 8)) and $01;
              PByteArray(DestScanline[y])^[x div 8] :=
               (PByteArray(DestScanline[y])^[x div 8] and
               not ($80 shr (x mod 8))) or
               (Bits shl (7 - x mod 8));
            end;
          end;
        end;
      end;
      Width := hh;
      Height := ww;
    end;
    Picture.Graphic := NewBitmap;
  finally
    NewBitmap.Free;
  end;

  AutoSize := asz;
  Stretch := srh;

  ReverseBitmap;
end;

function TLRBarImage.GetCDC: String;
var
  i, len: Integer;
begin
  Result := ''; // チェックデジットが存在しない場合の値

  if FBarType = cbarCUSTOMER then
  begin
    len := Length(FSymbol);
    if len > 0 then
      Result := FSymbol[len];
    Exit;
  end;

  i := Pos(FCode, FSymbol);
  if i = 0 then
    Exit;
  i := i + Length(FCode);
  len := Length(FSymbol) - (i - 1);
  if FBarType in [cbarCODE39, cbarNW7, cbarCODE93] then
    Dec(len);
  if len <= 0 then
    Exit;

  Result := Copy(FSymbol, i, len);
end;

procedure TLRBarImage.SetBarType(const Value: TBarTypes);
begin
  if not Stretch then
    begin
      if (FBarType = cbarCUSTOMER) and (Value <> cbarCUSTOMER) then
        begin
          if FScale > 1 then
            FScale := FScale div 2;
          FBarheight := FScale * 50;
        end
      else if (FBarType <> cbarCUSTOMER) and (Value = cbarCUSTOMER) then
        FScale := FScale * 2;
    end;
  FBarType := Value;
  PaintBarcode;
end;

procedure TLRBarImage.SetCode(const Value: String);
begin
  FCode := Value;
  PaintBarcode;
end;

procedure TLRBarImage.SetCommentUp(const Value: String);
begin
  FCommentUp := Value;
  PaintBarcode;
end;

procedure TLRBarImage.SetCommentUpLoc(const Value: TLocation);
begin
  FCommentUpLoc := Value;
  if FCommentUp <> '' then
    PaintBarcode;
end;

procedure TLRBarImage.SetCommentDown(const Value: String);
begin
  FCommentDown := Value;
  PaintBarcode;
end;

procedure TLRBarImage.SetCommentDownLoc(const Value: TLocation);
begin
  FCommentDownLoc := Value;
  if FCommentDown <> '' then
    PaintBarcode;
end;

procedure TLRBarImage.SetStartChar(const Value: Char);
begin
  if (Value >= 'A') and (Value <= 'D') then
    FStartChar := Value
  else
    FStartChar := Char('A');

  if (FBarType = cbarNW7) or (FBarType = cbarCODE128) then
    PaintBarcode;
end;

procedure TLRBarImage.SetStopChar(const Value: Char);
begin
  if (Value >= 'A') and (Value <= 'D') then
    FStopChar := Value
  else
    FStopChar := Char('A');

  if FBarType = cbarNW7 then
    PaintBarcode;
end;

procedure TLRBarImage.SetCD(const Value: Boolean);
begin
  FCD := Value;
  if (FBarType = cbarCODE39) or (FBarType = cbarNW7) or (FBarType = cbarITF) or
    (FBarType = cbarCTF) or (FBarType = cbarMATRIX) or (FBarType = cbarNEC) then
    PaintBarcode;
end;

procedure TLRBarImage.SetScale(const Value: TScale);
begin
  if (not Stretch) and (FBarType <> cbarCUSTOMER) then
    FBarHeight := FBarHeight * Value div FScale;
  FScale := Value;
  PaintBarcode;
end;

procedure TLRBarImage.SetAngle(const Value: Integer);
begin
  if (Value = 0) or (Value = 90) or (Value = 180) or (Value = 270) then
  begin
    FAngle := Value;
    PaintBarcode;
  end;
end;

procedure TLRBarImage.SetReverse(const Value: Boolean);
begin
  FReverse := Value;
  PaintBarcode;
end;

procedure TLRBarImage.SetSymbolDisp(const Value: Boolean);
begin
  FSymbolDisp := Value;
  PaintBarcode;
end;

procedure TLRBarImage.SetBarFont(const Value: TFont);
begin
  if (Value <> nil) and (Value <> FBarFont) then
  begin
    FBarFont.Assign(Value);
    if FSymbolDisp = True then
      PaintBarcode;
  end;
end;

procedure TLRBarImage.SetTransparent(const Value: Boolean);
begin
  FTransparent := Value;
  if FTransparent = True then
  begin
    if FMatch = True then
      FMatch := False;
    if Picture.Graphic is TJPEGImage then
      FBarPicture := picJPEG
    else
      FBarPicture := picBMP;
  end;
  PaintBarcode;
end;

procedure TLRBarImage.SetBarColor(const Value: TColor);
begin
  FBarColor := Value;
  PaintBarcode;
end;

procedure TLRBarImage.SetBackColor(const Value: TColor);
begin
  FBackColor := Value;
  PaintBarcode;
end;

procedure TLRBarImage.SetBarHeight(const Value: Integer);
begin
  FBarHeight := Value;
  if FBarType <> cbarCUSTOMER then
    PaintBarcode;
end;

procedure TLRBarImage.SetBarLeft(const Value: Integer);
begin
  FBarLeft := Value;
  PaintBarcode;
end;

procedure TLRBarImage.SetBarTop(const Value: Integer);
begin
  FBarTop := Value;
  PaintBarcode;
end;

procedure TLRBarImage.SetBarSpaceUp(const Value: Integer);
begin
  if Value < 0 then
    FBarSpaceUp := 0
  else
    FBarSpaceUp := Value;
  PaintBarcode;
end;

procedure TLRBarImage.SetBarSpaceDown(const Value: Integer);
begin
  if Value < 0 then
    FBarSpaceDown := 0
  else
    FBarSpaceDown := Value;
  PaintBarcode;
end;

procedure TLRBarImage.SetBarSpaceLeft(const Value: Integer);
begin
  if Value < 0 then
    FBarSpaceLeft := 0
  else
    FBarSpaceLeft := Value;
  PaintBarcode;
end;

procedure TLRBarImage.SetBarSpaceRight(const Value: Integer);
begin
  if Value < 0 then
    FBarSpaceRight := 0
  else
    FBarSpaceRight := Value;
  PaintBarcode;
end;

procedure TLRBarImage.SetMatch(const Value: Boolean);
begin
  FMatch := Value;
  if FMatch = True then
  begin
    if FTransparent = True then
      FTransparent := False;
  end
  else
  begin
    if Picture.Graphic is TJPEGImage then
      FBarPicture := picJPEG
    else
      FBarPicture := picBMP;
  end;
  PaintBarcode;
end;

procedure TLRBarImage.SetBarPicture(const Value: TPictures);
begin
  if FMatch = False then
  begin
    if Value in [picBMP, picJPEG] then
      FBarPicture := Value
    else
      FBarPicture := picBMP;
  end
  else
    FBarPicture := Value;
  PaintBarcode;
end;

procedure TLRBarImage.SetElementRatio(const Value: TElementRatio);
begin
  FElementRatio := Value;
  if (FBarType >= cbarCODE39) and (FBarType <= cbarNEC) then
    PaintBarcode;
end;

procedure TLRBarImage.SetBarEnabled(const Value: Boolean);
begin
  if Value = False then
    FBarEnabled := Value
  else
  begin
    if FBarEnabled = False then
    begin
      FBarEnabled := Value;
      PaintBarcode;
    end;
  end;
end;

procedure TLRBarImage.SetBarDebug(const Value: Boolean);
begin
  FBarDebug := Value
end;

procedure TLRBarImage.SetBarFrame(const Value: TFrameSize);
begin
  FBarFrame := Value;
  PaintBarcode;
end;

procedure TLRBarImage.DrawCUSTOMER(const Position, Cnt, Scl, Style: Integer);
var
  i, j: Integer;
  X1,
  Y1,
  X2,
  Y2: Integer;

begin
  for i := 2 downto 0 do
  begin
    j := 1 shl (2 - i);
    if (j and Style) = j then
    begin
      X1 := FBarLeft + FBarSpaceLeft + Position + (Cnt * Scl) - Scl;
      Y1 := FBarTop + FBarSpaceUp + ((i + 1) * (Scl * 2)) - (Scl * 2);
      X2 := FBarLeft + FBarSpaceLeft + Position + (Cnt * Scl);
      Y2 := FBarTop + FBarSpaceUp + ((i + 1) * (Scl * 2));
      if FBarPicture = picBMP then
        Picture.Bitmap.Canvas.FillRect(Rect(X1, Y1, X2, Y2))
      else
        FMfCanvas.FillRect(Rect(X1, Y1, X2, Y2));
    end;
  end;
end;

function TLRBarImage.WidthCUSTOMER(const Module: String): Integer;
var
  i, j,
  X1, X2: Integer;
  Cnt,
  Style: Integer;
  done: Boolean;

begin
  done := False;
  X1 := 0;
  X2 := 0;

  for Cnt := 0 to Length(Module) - 1 do
  begin
    Style := StrToInt(Module[Cnt + 1]);
    for i := 2 downto 0 do
    begin
      j := 1 shl (2 - i);
      if (j and Style) = j then
      begin
        if done = False then
        begin
          done := True;
          X1 := (Cnt * FScale) - FScale;
        end;
        X2 := Cnt * FScale;
      end;
    end;
  end;

  if done = True then
    Result := X2 - X1 + FScale
  else
    Result := 0;
end;

{ UPC-E 形式のバーコード文字列(7桁)を UPC-A 形式のバーコード文字列(11桁) }
{ に変換する関数 }
function TLRBarImage.UPCE2A(const Str: String): String;
var
  upca,
  upce: String;

begin
  upce := Str;

  if (Length(upce) <> 7) or (upce[1] <> '0') then
    upca := ''
  else if (Pos(upce[7], '56789') <> 0) and (upce[6] <> '0') then
    upca := Copy(upce, 1, 6) + '0000' + upce[7]
  else if (upce[7] = '4') and (upce[5] <> '0') then
    upca := Copy(upce, 1, 5) + '00000' + upce[6]
  else if (upce[7] = '3') and  (Pos(upce[4], '3456789') <> 0) then
    upca := Copy(upce, 1, 4) + '00000' + Copy(upce, 5, 2)
  else if (Pos(upce[7], '012') <> 0) then
    upca := Copy(upce, 1, 3) + upce[7] + '0000' + Copy(upce, 4, 3)
  else
    upca := '';

  Result := upca;
end;

procedure TLRBarImage.DrawFrame(const fc: TCanvas; const r: TRect);
var
  lx, ly, rx, ry, g: Integer;

begin
  lx := r.left;
  ly := r.top;
  rx := r.right;
  ry := r.bottom;

  g := (ry - ly) div 2;
  if g > (rx - lx) div 2 then
    g := (rx - lx) div 2;
  if g > 0 then
    Dec(g);
  if g < 1 then
  begin
    FBarFrame := 0;  // フレームを描けない場合は、強制的に 0 に設定します。
    Exit;
  end;

  if FBarFrame > g then
    FBarFrame := g;

  fc.Brush.Color := FBarColor;
  fc.Brush.Style := bsSolid;
  fc.FillRect(Rect(lx, ly, rx, ly + FBarFrame));
  fc.FillRect(Rect(lx, ry - FBarFrame, rx, ry));
  fc.FillRect(Rect(lx, ly + FBarFrame, lx + FBarFrame, ry - FBarFrame));
  fc.FillRect(Rect(rx - FBarFrame, ly + FBarFrame, rx, ry- FBarFrame));
  fc.Brush.Color := FBackColor;
  fc.Brush.Style := bsClear;
end;

end.


