﻿{
  TLRDBItem
    TLRItemから派生
    データベース対応コンポーネント
    指定されたデータ項目から DisplayText の内容をセットする
  プロパティ
    DataField: string  フィールド名
    DataSource: TDataSource  データソース

  TLRDBItemR
    TLRItemRから派生
    データベース対応コンポーネントの角丸対応
    指定されたデータ項目から DisplayText の内容をセットする
    角丸表示のロジックが複雑と思うため、角丸を持たないコンポと２種類にした
  プロパティ
    DataField: string  フィールド名
    DataSource: TDataSource  データソース

  Ver1.00 の TLRDBItem , TLRDBItemR を使用されている場合 DataSet と FieldName
  プロパティが無くなっています。
  DataSource と DataField プロパティに設定し直してください。m(__)m

}

{
  2000.09.07 LRSuppl から DB対応コンポを別ユニットにした
             ... DB 関連を使用しない場合の .exe の容量に配慮するため

}
unit LRDBSpl;

interface

uses
  Windows, SysUtils, Classes, Graphics, LightRep, LRSuppl, Db, DBCtrls;

type
  // データベース対応コンポーネント
  TLRDBItem = class(TLRItem)
  private
    FDataLink: TFieldDataLink;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(const Value: TDataSource);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure ActiveChange(Sender: TObject); virtual;
    procedure DataChange(Sender: TObject); virtual;
    procedure Print(ACanvas: TCanvas; ARect: TRect;
      APrintScale: Double; APrintStatus: TPrintStatus); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

  // データベース対応コンポーネント(角丸)
  TLRDBItemR = class(TLRItemR)
  private
    FDataLink: TFieldDataLink;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(const Value: TDataSource);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure ActiveChange(Sender: TObject); virtual;
    procedure DataChange(Sender: TObject); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
  end;

implementation

{ TLRDBItem }

constructor TLRDBItem.Create(AOwner: TComponent);
begin
  inherited;
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnActiveChange := ActiveChange;
end;

destructor TLRDBItem.Destroy;
begin
  FDataLink.Free;
  inherited;
end;

procedure TLRDBItem.ActiveChange(Sender: TObject);
begin
  if  csDesigning in ComponentState  then  // 設計時のみフィールド名or名前をセットする
    begin
      if  not FDataLink.DataSet.Active  then
        if  FDataLink.FieldName = ''  then  Text := Name
        else  Text := FDataLink.FieldName;
    end;
end;

procedure TLRDBItem.DataChange(Sender: TObject);
var
  Field: TField;
begin
  if  FDataLink.FieldName <> ''  then
    begin
      Field := FDataLink.DataSet.FindField(FDataLink.FieldName);
      if  Assigned(Field)  then  Text := Field.DisplayText;
    end;
end;

procedure TLRDBItem.SetDataSource(const Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

function TLRDBItem.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TLRDBItem.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
  if  Value <> ''  then  Text := Value;
end;

function TLRDBItem.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

procedure TLRDBItem.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if  (Operation = opRemove) and (AComponent = FDataLink.DataSource)  then
    FDataLink.DataSource := nil;
end;

procedure TLRDBItem.Print(ACanvas: TCanvas; ARect: TRect;
  APrintScale: Double; APrintStatus: TPrintStatus);
begin
  inherited;
{  if  csDesigning in ComponentState  then
    DrowDesignFrame(ACanvas, Color, ARect,
      LineLeft, LineTop, LineRight, LineBottom);}
end;

{ TLRDBItemR }

constructor TLRDBItemR.Create(AOwner: TComponent);
begin
  inherited;
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnActiveChange := ActiveChange;
end;

destructor TLRDBItemR.Destroy;
begin
  FDataLink.Free;
  inherited;
end;

procedure TLRDBItemR.ActiveChange(Sender: TObject);
begin
  if  not FDataLink.DataSet.Active  then
    if  FDataLink.FieldName = ''  then  Text := Name
    else  Text := FDataLink.FieldName;
end;

procedure TLRDBItemR.DataChange(Sender: TObject);
var
  Field: TField;
begin
  if  FDataLink.FieldName <> ''  then
    begin
      Field := FDataLink.DataSet.FindField(FDataLink.FieldName);
      if  Assigned(Field)  then  Text := Field.DisplayText;
    end;
end;

procedure TLRDBItemR.SetDataSource(const Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

function TLRDBItemR.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TLRDBItemR.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
  if  Value <> ''  then  Text := Value;
end;

function TLRDBItemR.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

procedure TLRDBItemR.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if  (Operation = opRemove) and (AComponent = FDataLink.DataSource)  then
    FDataLink.DataSource := nil;
end;

end.
