﻿{
  TLRPreviewControl
    プレビューコントロールコンポーネント
    プリビューで前後出来るように、リストに表示イメージを保存、管理します。
    プレビューを表示するフォームは持っていません、コントロールのみです。
}

{
  2000/09/15 紆余曲折を経て新規作成
  2001/10/17 プレビュー表示の色がおかしくなる不具合を修正
  2001/10/25 プレビューイメージの保存に TNkDIB を使用するよう変更
             ZLib は不要になりました
             が、TNkDIB を既にご使用の場合は問題が発生します tnkdib.txt
             を参照してください
  2001/10/26 プレビュー表示が色数に大きく左右されるため PixelFormat
             プロパティを追加
             表示イメージの保存を TMemoryStream から TFileStream に変更
             SmallPreview の場合表示イメージ内に複数ページが表示出来るため
             PageNumber で指定できる ShowPageNumberメソッドを追加
             PageNumberプロパティはややこしくなるので Pageプロパティに変更
  2003/05/12 メモリリークが発覚^^;
             Destroy の中の以下 2点を修正
             FBitmapList.Clear;  -> FBitmapList.Free;
             FPageNumList.Clear; -> FPageNumList.Free;
}

unit LRPrvSpl;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, ExtCtrls, LightRep, NkDIB;

type
  // プレビューコントロールコンポーネント
  TLRPreviewControl = class(TComponent)
  private
    { Private 宣言 }
    FLightRep: TLightRep;
    FBitmapList, FPageNumList: TList;
    FBitmap: TBitmap;
    FPixelFormat: TPixelFormat;
    FIndex: Integer;
    FImage: TImage;
    FBitmapFileName: string;
    FBitmapStream: TFileStream;
    function GetPage: Integer;
    function GetBop: Boolean;
    function GetEop: Boolean;
    function GetTotalPages: Integer;
    function UniqueTempFileName: string;
    procedure SetLightRep(const Value: TLightRep);
    procedure CreateFCanvas;
  protected
    { protected 宣言 }
    function AddImage(ABitmap: TBitmap): Integer;
    function AddBitmapListIndex: Integer;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure ShowImage(AIndex: Integer);
  public
    { Public 宣言 }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Next: Boolean;
    procedure BeginPreview(Image: TImage);
    procedure EndPreview;
    procedure Preview;
    procedure Prior;
    procedure ShowPageNumber(APageNumber: Integer);
    procedure ShowPage(AIndex: Integer);
    procedure ShowFirstPage;
    procedure ShowLastPage;
    procedure SmallPreview(X, Y: Integer; Scale: Integer);
    procedure SmallPreviewSuspend;
    procedure ThreadPreview;
    procedure ThreadSmallPreview(X, Y: Integer; Scale: Integer);
    procedure ThreadSmallPreviewSuspend;
    property Index: Integer read FIndex;
    property Page: Integer read GetPage;
    property TotalPages: Integer read GetTotalPages;
    property Bop: Boolean read GetBop;
    property Eop: Boolean read GetEop;
  published
    { Published 宣言 }
    property LightRep: TLightRep read FLightRep write SetLightRep;
    property PixelFormat: TPixelFormat read FPixelFormat write FPixelFormat default pf8bit;
  end;

implementation

{ TLRPreviewControl }

constructor TLRPreviewControl.Create(AOwner: TComponent);
begin
  inherited;
  //  初期値は 256色とする
  FPixelFormat := pf8bit;
  //  TNkDIB をストリームに保管した開始アドレスを記録する
  FBitmapList := TList.Create;
  //  PageNumber を含むFBitmapListのIndexを記録する
  FPageNumList := TList.Create;
end;

destructor TLRPreviewControl.Destroy;
var
  i: Integer;
begin
  for  i := FBitmapList.Count - 1 downto 0  do
    Dispose(FBitmapList[i]);
  FBitmapList.Free;
  for  i := FPageNumList.Count - 1 downto 0  do
    Dispose(FPageNumList[i]);
  FPageNumList.Free;
  if  Assigned(FBitmapStream)  then
    begin
      FBitmapStream.Destroy;
      DeleteFile(FBitmapFileName);
    end;
  if  Assigned(FBitmap)  then  FBitmap.Free;
  inherited;
end;

procedure TLRPreviewControl.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if  (Operation = opRemove) and (AComponent = FLightRep)  then
    LightRep := nil;
end;

procedure TLRPreviewControl.CreateFCanvas;
begin
  // 指定された色数でプレビュー用の Bitmap を作成する
  if  not Assigned(FBitmap)  then
    begin
      FBitmap := TBitmap.Create;
      FBitmap.PixelFormat := FPixelFormat;  // 色数の指定
      FBitmap.Height := FImage.Height;
      FBitmap.Width := FImage.Width;
    end;
  with  FBitmap.Canvas  do
  begin
    Brush.Color := clWhite;
    FillRect(Rect(0, 0, FBitmap.Width - 1, FBitmap.Height - 1));
  end;
end;

procedure TLRPreviewControl.BeginPreview(Image: TImage);
var
  i: Integer;
begin
  //  前回のプレビューの後始末
  for  i := FBitmapList.Count - 1 downto 0  do
    Dispose(FBitmapList[i]);
  FBitmapList.Clear;
  for  i := FPageNumList.Count - 1 downto 0  do
    Dispose(FPageNumList[i]);
  FPageNumList.Clear;
  //  一時ファイルを削除する
  if  Assigned(FBitmapStream)  then
    begin
      FBitmapStream.Destroy;
      DeleteFile(FBitmapFileName);
    end;
  // Tempファイル名の取得
  FBitmapFileName := UniqueTempFileName;
  //  イメージ保存用の一時ファイルを確保
  //  システムのTempフォルダに vLR[連番].txt の形式で作成
  FBitmapStream := TFileStream.Create(FBitmapFileName,
    fmCreate or fmShareExclusive);
  //  プレビュー用の前準備を行う
  FImage := Image;
  CreateFCanvas;
  LightRep.BeginPreview;
end;

procedure TLRPreviewControl.EndPreview;
begin
  LightRep.EndPreview;
end;

function TLRPreviewControl.AddImage(ABitmap: TBitmap): Integer;
var
  TempNkDIB: TNkDIB;
  pPos: ^Integer;
begin
  //  表示用に描画した後、RLE圧縮(NkDIB)して保管
  TempNkDIB := TNkDIB.Create;
  try
    TempNkDIB.Assign(ABitmap);
    try
    if  FPixelFormat = pf4Bit  then
      TempNkDIB.PixelFormat := nkPf4BitRLE
    else
      TempNkDIB.PixelFormat := nkPf8BitRLE;
    except
      on ENkDIBCompressionFailed  do
    end;
    //  メインスレッドから要求があった場合Positionがずれるので最後に位置づける
    FBitmapStream.Position := FBitmapStream.Size;
    New(pPos);
    pPos^ := FBitmapStream.Position;
    TempNkDIB.SaveToStream(FBitmapStream);
    Result := FBitmapList.Add(pPos);
  finally
    TempNkDIB.Free;
  end;
end;

function TLRPreviewControl.AddBitmapListIndex: Integer;
var
  pPos: ^Integer;
begin
  New(pPos);
  pPos^ := FBitmapList.Count;
  Result := FPageNumList.Add(pPos);
end;

procedure TLRPreviewControl.ShowImage(AIndex: Integer);
var
  TempNkDIB: TNkDIB;
begin
  //  保管していた画像を解凍して表示用に描画
  TempNkDIB := TNkDIB.Create;
  try
    FBitmapStream.Position := Integer(FBitmapList[AIndex]^);
    TempNkDIB.LoadFromStream(FBitmapStream);
    FImage.Picture.Graphic := TempNkDIB;
    FIndex := AIndex;
  finally
    TempNkDIB.Free;
  end;
end;

function TLRPreviewControl.GetPage: Integer;
begin
  Result := FIndex + 1;
end;

function TLRPreviewControl.GetTotalPages: Integer;
begin
  Result := FBitmapList.Count;
end;

function TLRPreviewControl.Next: Boolean;
begin
  //  次のページを一度表示していたら保管リストから表示する
  Result := (FBitmapList.Count - 1) > FIndex;
  if  Result  then
    begin
      Inc(FIndex);
      ShowImage(FIndex);
    end;
end;

procedure TLRPreviewControl.Preview;
begin
  //  次のページが初めての表示の場合 LightReport2に描画をお願いして保管する
  if  not Next  then
    begin
      LightRep.Preview(FBitmap.Canvas);
      AddBitmapListIndex;
      ShowImage(AddImage(FBitmap));
    end;
end;

procedure TLRPreviewControl.Prior;
begin
  //  前のページを保管リストから表示する
  if  FIndex > 0  then
    begin
      Dec(FIndex);
      ShowImage(FIndex);
    end;
end;

procedure TLRPreviewControl.SetLightRep(const Value: TLightRep);
begin
  FLightRep := Value;
end;

procedure TLRPreviewControl.SmallPreview(X, Y, Scale: Integer);
begin
  //  縮小表示の場合は１ページの区切りはアプリケーション側で判断する
  //  ここではサムネイル１つの描画のみ
  LightRep.ScaledPreview(FBitmap.Canvas, X, Y, Scale, FPixelFormat);
  FImage.Picture.Graphic := FBitmap;
  FImage.Repaint;
  AddBitmapListIndex;
end;

procedure TLRPreviewControl.SmallPreviewSuspend;
begin
  //  ページ区切りでリストに保管する
  ShowImage(AddImage(FBitmap));
  //  描画用のCanvas を再作成
  CreateFCanvas;
end;

// スレッド処理用のプレビュー
procedure TLRPreviewControl.ThreadPreview;
begin
  LightRep.Preview(FBitmap.Canvas);
  AddBitmapListIndex;
  AddImage(FBitmap);
end;

// スレッド処理用の縮小プレビュー
procedure TLRPreviewControl.ThreadSmallPreview(X, Y, Scale: Integer);
begin
  //  縮小表示の場合は１ページの区切りはアプリケーション側で判断する
  //  ここではサムネイル１つの描画のみ
  LightRep.ScaledPreview(FBitmap.Canvas, X, Y, Scale, FPixelFormat);
  AddBitmapListIndex;
end;

// スレッド処理用の縮小プレビュー一時停止
procedure TLRPreviewControl.ThreadSmallPreviewSuspend;
begin
  //  ページ区切りでリストに保管する
  AddImage(FBitmap);
  //  描画用のCanvas を再作成
  CreateFCanvas;
end;

function TLRPreviewControl.GetBop: Boolean;
begin
  //  最初のページ
  Result := FIndex = 0;
end;

function TLRPreviewControl.GetEop: Boolean;
begin
  //  リストに入っている最終のページ
  Result := (FBitmapList.Count = 0) or ((FBitmapList.Count - 1) = FIndex);
end;

procedure TLRPreviewControl.ShowPageNumber(APageNumber: Integer);
begin
  //  保管リストから指定ページを表示する
  if  (APageNumber < 1) or (APageNumber > FPageNumList.Count)  then
    Beep
  else
    begin
      FIndex := Integer(FPageNumList[APageNumber - 1]^);
      ShowImage(FIndex);
    end;
end;

procedure TLRPreviewControl.ShowPage(AIndex: Integer);
begin
  //  保管リストから指定ページを表示する
  if  (AIndex < 1) or (AIndex > FBitmapList.Count)  then
    Beep
  else
    begin
      FIndex := AIndex - 1;
      ShowImage(FIndex);
    end;
end;

procedure TLRPreviewControl.ShowFirstPage;
begin
  ShowPage(1);
end;

procedure TLRPreviewControl.ShowLastPage;
begin
  ShowPage(FBitmapList.Count);
end;

function TLRPreviewControl.UniqueTempFileName: string;
var
  i: Integer;
  TmpPath, TmpFileName: string;
begin
  // Tempファイル名の取得
  // DelphiML にて戸田 英夫さんからご教授頂きました(感謝)
  i := 1;
  Result := '';
  while  Result = ''  do
  begin
    SetLength(TmpPath, MAX_PATH - 12);
    SetLength(TmpFileName, MAX_PATH + 1);
    GetTempPath(Length(TmpPath), PChar(TmpPath));
    GetTempFileName(PChar(TmpPath), 'vLR', i, PChar(TmpFileName));
    SetLength(TmpFileName, StrLen(PChar(TmpFileName)));
    if  FileExists(TmpFileName)  then    // 存在チェック
      if  DeleteFile(TmpFileName)  then  // 排他モードで開いているので削除出来ない
        Result := TmpFileName            // 削除出来る場合は強制終了などでワークファイル
      else                               // が残っているものとしてファイル名を利用する
    else
      Result := TmpFileName;
    Inc(i);
  end;
end;

end.
