﻿{
  TLRItemR
    TLRItemから派生
    枠を角丸にするコンポーネント
    角丸の先の太さは RoundLeftTop, RoundRightTopが LineTop に
    RoundLeftBottom, RoundRightBottomが LineBottom に依存する
  プロパティ
    RoundSize: Integer  角の丸みの大きさ
    RoundLeftTop: Boolean  左上角丸
    RoundLeftBottom: Boolean  左下角丸
    RoundRightTop: Boolean  右上角丸
    RoundRightBottom: Boolean  右下角丸
    DiagonalRisedLeft: TLineStyle  左上がりの対角線
    DiagonalRisedRight: TLineStyle  右上がりの対角線
}

{
  2000.09.07 LRSuppl から DB対応コンポを別ユニット(LRDBSpl) にした
             ... DB 関連を使用しない場合の .exe の容量に配慮するため
  2001/02/07 TLRItemR に対角線を引けるように機能追加
             以下のプロパティを追加
             DiagonalRisedLeft: TLineStyle  左上がりの対角線
             DiagonalRisedRight: TLineStyle  右上がりの対角線
}

unit LRSuppl;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Mask, MaskUtils, LightRep, LRConsts;

type
  // 枠の角丸対応コンポーネント
  TLRItemR = class(TLRItem)
  private
    FRoundSize: Integer;
    FRoundRect: array[0..3] of Boolean;
    FDiagonalRisedLeft: TLineStyle;
    FDiagonalRisedRight: TLineStyle;
    procedure SetRoundSize(Value: Integer);
    procedure SetLineStyle(Canvas: TCanvas; Style: TLineStyle; PrintScale: double);
    procedure SetRoundRect(const Index: Integer; const Value: Boolean);
    procedure SetDiagonalRisedLeft(const Value: TLineStyle);
    procedure SetDiagonalRisedRight(const Value: TLineStyle);
  protected
    function InsidePoint(ABool: Boolean; X, Y, XOffset, YOffset: Integer): TPoint;
    function DisplayText: string;
    procedure Print(ACanvas: TCanvas; ARect: TRect;
      APrintScale: Double; APrintStatus: TPrintStatus); override;
    procedure PrintFrame(ACanvas: TCanvas; ARect: TRect;
      APrintScale: Double); virtual;
    procedure PrintText(ACanvas: TCanvas; ARect: TRect;
      APrintScale: Double); virtual;
    procedure CornerOfRound(Canvas: TCanvas; Style: TLineStyle; PrintScale: Double;
      X1, Y1, X2, Y2, X3, Y3, X4, Y4, XOffset, YOffset: Integer);
  public
  published
    property DiagonalRisedLeft: TLineStyle read FDiagonalRisedLeft write SetDiagonalRisedLeft;
    property DiagonalRisedRight: TLineStyle read FDiagonalRisedRight write SetDiagonalRisedRight;
    property RoundSize: Integer read FRoundSize write SetRoundSize;
    property RoundLeftTop: Boolean index 0 read FRoundRect[0] write SetRoundRect;
    property RoundLeftBottom: Boolean index 1 read FRoundRect[1] write SetRoundRect;
    property RoundRightTop: Boolean index 2 read FRoundRect[2] write SetRoundRect;
    property RoundRightBottom: Boolean index 3 read FRoundRect[3] write SetRoundRect;
  end;

implementation

{ TLRItemR }

procedure TLRItemR.SetRoundSize(Value: Integer);
begin
  if  (0 > Value) or (20 < Value)  then
    raise Exception.Create(LRERR_INVALID_ROUNDSIZE);
  if Value <> FRoundSize then
    begin
      FRoundSize := Value;
      Invalidate;
    end;
end;

procedure TLRItemR.SetRoundRect(const Index: Integer; const Value: Boolean);
begin
  if  FRoundRect[Index] <> Value  then
    begin
      FRoundRect[Index] := Value;
      Invalidate;
    end;
end;

procedure TLRItemR.Print(ACanvas: TCanvas; ARect: TRect;
  APrintScale: Double; APrintStatus: TPrintStatus);
begin
  if  APrintStatus in [psNormal, psLineOnly]  then
    begin
      PrintFrame(ACanvas, ARect, APrintScale);  // 枠線を引いて
      if  APrintStatus = psNormal  then
        PrintText(ACanvas, ARect, APrintScale); // テキストを書く
    end;
  if  csDesigning in ComponentState  then       // 設計時枠線を書く
    DrowDesignFrame(ACanvas, Color, ARect,
      LineLeft, LineTop, LineRight, LineBottom);
end;

procedure TLRItemR.PrintFrame(ACanvas: TCanvas; ARect: TRect;
  APrintScale: Double);
var
  PrintRect: TRect;
  InsideLeftTop, InsideRightTop, InsideLeftBottom, InsideRightBottom: TPoint;
  ArcInside: TPoint;
  ARoundSize, Offset, OffsetIntercept: Integer;
begin
  // 印刷面のスケールに変換する
  with  ARect  do
    PrintRect := PrintScaleRect(Rect(Left, Top, Right-1, Bottom-1), APrintScale);
  ARoundSize := ToPrintScale(FRoundSize, APrintScale);
  // 枠線オフセットの計算
  if  csDesigning in ComponentState  then  Offset := 0
  else
    begin
      Offset := Trunc(APrintScale);
      PrintRect.Left := PrintRect.Left - Trunc(APrintScale);
      PrintRect.Top := PrintRect.Top - Trunc(APrintScale);
    end;
  // 角丸用にLine を短くする
  InsideLeftTop := InsidePoint(RoundLeftTop, PrintRect.Left,
    PrintRect.Top, ARoundSize, ARoundSize);
  InsideRightTop := InsidePoint(RoundRightTop, PrintRect.Right,
    PrintRect.Top, -ARoundSize, ARoundSize);
  InsideLeftBottom := InsidePoint(RoundLeftBottom, PrintRect.Left,
    PrintRect.Bottom, ARoundSize, -ARoundSize);
  InsideRightBottom := InsidePoint(RoundRightBottom, PrintRect.Right,
    PrintRect.Bottom, -ARoundSize, -ARoundSize);
  with  ACanvas  do
  begin
  // 枠内を塗る
    if  TransParent  then  Brush.Style := bsClear
    else
      begin
        Brush.Color := Color;                                //      c-----b
        Pen.Style := psClear;                                //      |     |
        Polygon([InsideLeftTop,                              // 1  2-1     a-9
          Point(PrintRect.Left + Offset, InsideLeftTop.Y),   // 2  |         |
          Point(PrintRect.Left + Offset, InsideLeftBottom.Y),// 3  |         |
          InsideLeftBottom,                                  // 4  3-4     7-8
          Point(InsideLeftBottom.X, PrintRect.Bottom),       // 5    |     |
          Point(InsideRightBottom.X, PrintRect.Bottom),      // 6    5-----6
          InsideRightBottom,                                 // 7  上記のような矩形を塗っている
          Point(PrintRect.Right, InsideRightBottom.Y),       // 8  角丸がない場合は1, 4, 7, a の値が
          Point(PrintRect.Right, InsideRightTop.Y),          // 9  PrintRect の値と同じなので
          InsideRightTop,                                    // a      1-c--
          Point(InsideRightTop.X, PrintRect.Top + Offset),   // b      |
          Point(InsideLeftTop.X, PrintRect.Top + Offset)]);  // c      2    のように角がへこまない
      end;                                                   //        |
  // 枠線の描写
    Pen.Color := LineColor;
    Pen.Style := psSolid;
    if  LineTop > lsNone  then
    begin
      SetLineStyle(ACanvas, LineTop, APrintScale);
      MoveTo(InsideLeftTop.X, PrintRect.Top);
      LineTo(InsideRightTop.X, PrintRect.Top);
    end;
    if  LineLeft > lsNone  then
    begin
      SetLineStyle(ACanvas, LineLeft, APrintScale);
      MoveTo(PrintRect.Left, InsideLeftTop.Y);
      LineTo(PrintRect.Left, InsideLeftBottom.Y);
    end;
    if  LineRight > lsNone  then
    begin
      SetLineStyle(ACanvas, LineRight, APrintScale);
      if  RoundRightTop  then
        MoveTo(PrintRect.Right, InsideRightTop.Y - Offset)
      else
        MoveTo(PrintRect.Right, InsideRightTop.Y);
      LineTo(PrintRect.Right, InsideRightBottom.Y);
    end;
    if  LineBottom > lsNone  then
    begin
      SetLineStyle(ACanvas, LineBottom, APrintScale);
      if  RoundLeftBottom  then
        MoveTo(InsideLeftBottom.X - Offset, PrintRect.Bottom)
      else
        MoveTo(InsideLeftBottom.X, PrintRect.Bottom);
      LineTo(InsideRightBottom.X, PrintRect.Bottom);
    end;
  end;
  // 左上角丸
  if  RoundLeftTop  then
    begin
      // 角丸用にArc 用の内側の位置を計算
      ArcInside := InsidePoint(RoundLeftTop, PrintRect.Left + 1,
        PrintRect.Top + 1, ARoundSize * 2, ARoundSize * 2);
      // 角丸を描く
      CornerOfRound(ACanvas, LineTop, APrintScale,
        PrintRect.Left, PrintRect.Top, ArcInside.X, ArcInside.Y,
        InsideLeftTop.X, PrintRect.Top, PrintRect.Left, InsideLeftTop.Y,
        Offset, Offset);
    end;
  // 右上角丸
  if  RoundRightTop  then
    begin
      ArcInside := InsidePoint(RoundRightTop, PrintRect.Right - 1,
        PrintRect.Top + 1, -ARoundSize * 2, ARoundSize * 2);
      CornerOfRound(ACanvas, LineTop, APrintScale,
        ArcInside.X, PrintRect.Top, PrintRect.Right, ArcInside.Y,
        PrintRect.Right, InsideRightTop.Y, InsideRightTop.X, PrintRect.Top,
        0, Offset);
    end;
  // 左下角丸
  if  RoundLeftBottom  then
    begin
      ArcInside := InsidePoint(RoundLeftBottom, PrintRect.Left + 1,
        PrintRect.Bottom - 1, ARoundSize * 2, -ARoundSize * 2);
      CornerOfRound(ACanvas, LineBottom, APrintScale,
        PrintRect.Left, ArcInside.Y, ArcInside.X, PrintRect.Bottom,
        PrintRect.Left, InsideLeftBottom.Y, InsideLeftBottom.X, PrintRect.Bottom,
        Offset, 0);
    end;
  // 右下角丸
  if  RoundRightBottom  then
    begin
      ArcInside := InsidePoint(RoundRightBottom, PrintRect.Right - 1,
        PrintRect.Bottom - 1, -ARoundSize * 2, -ARoundSize * 2);
      CornerOfRound(ACanvas, LineBottom, APrintScale,
        ArcInside.X, ArcInside.Y, PrintRect.Right, PrintRect.Bottom,
        InsideRightBottom.X, PrintRect.Bottom, PrintRect.Right, InsideRightBottom.Y,
        0, 0);
    end;
  // 対角線
  // 角丸の切片用のオフセットを求める
  if  ARoundSize = 0  then
    OffsetIntercept := 0
  else
    OffsetIntercept := Trunc(Sqr(ARoundSize) / Sqrt(Sqr(ARoundSize) * 2)) - 1; // ちょっと内側
  with  ACanvas  do
  begin
    // 左上がりの対角線
    if  DiagonalRisedLeft > lsNone  then
      begin
        SetLineStyle(ACanvas, DiagonalRisedLeft, APrintScale);
        if  RoundLeftTop  then
          MoveTo(InsideLeftTop.X - OffsetIntercept, InsideLeftTop.Y - OffsetIntercept)
        else
          MoveTo(PrintRect.Left, PrintRect.Top);
        if  RoundRightBottom  then
          LineTo(InsideRightBottom.X + OffsetIntercept, InsideRightBottom.Y + OffsetIntercept)
        else
          LineTo(PrintRect.Right, PrintRect.Bottom);
      end;
    // 右上がりの対角線
    if  DiagonalRisedRight > lsNone  then
      begin
        SetLineStyle(ACanvas, DiagonalRisedRight, APrintScale);
        if  RoundLeftBottom  then
          MoveTo(InsideLeftBottom.X - OffsetIntercept, InsideLeftBottom.Y + OffsetIntercept)
        else
          MoveTo(PrintRect.Left, PrintRect.Bottom);
        if  RoundRightTop  then
          LineTo(InsideRightTop.X + OffsetIntercept, InsideRightTop.Y - OffsetIntercept)
        else
          LineTo(PrintRect.Right, PrintRect.Top);
      end;
  end;
end;

procedure TLRItemR.SetLineStyle(Canvas: TCanvas; Style: TLineStyle;
  PrintScale: double);
begin
  // Style に応じて、画面上の線の太さを変える
  // プライベートなので再定義(Ohtaka)
  if (csDesigning in ComponentState) then
    case Style of
      lsHalf: Canvas.Pen.Color := clSilver;
      lsSingle: Canvas.Pen.Color := clGray;
      lsDouble: Canvas.Pen.Color := clBlack;
    end
  else
    case Style of
      lsHalf: Canvas.Pen.Width := Trunc(PrintScale / 2);
      lsSingle: Canvas.Pen.Width := Trunc(PrintScale * 1.2);
      lsDouble: Canvas.Pen.Width := Trunc(PrintScale * 2);
    end;
end;

function TLRItemR.InsidePoint(ABool: Boolean; X, Y,
  XOffset, YOffset: Integer): TPoint;
begin
  // 角丸用にLine を短くするための位置を計算
  if  ABool  then
    begin
      Result.X := X + XOffset;
      Result.Y := Y + YOffset;
    end
  else
    begin
      Result.X := X;
      Result.Y := Y;
    end;
end;

procedure TLRItemR.CornerOfRound(Canvas: TCanvas; Style: TLineStyle;
   PrintScale: Double; X1, Y1, X2, Y2, X3, Y3, X4, Y4, XOffset, YOffset: Integer);
begin
  // 各角丸を書く
  // X1, X1, Y1, X2, Y2, X3, Y3, X4, Y4 については
  // TCanvas.Pie と TCanvas.Arc のHelp を参照のこと


  //  1=(X1, Y1) 2=(X2, Y2)
  //  3=(X3, Y3) 4=(X4, Y4)              1 と 2 で表す正方形(A)に接する円の一部
  //                                     を書くことで角丸を表現
  //  左上    右上    左下   左上        正方形(A)の二辺は角丸サイズ(RoundSize)
  //  1..3--- 1--4... 1...... 1.....|    の倍をとる
  //  .／   . .   ＼. |     . .     |    時計の逆回りで円の一部が描かれる
  //  4     . .     3 3     . .     4    3 と 4 の位置は正方形(A)の中間の位置を
  //  |     . .     | .＼   . .   ／.    表している
  //  |.....2 ......2 ...4--2 ---3..2    先に扇形の部分を塗って、弧の部分だけの
  //                                     線を書いている
  with  Canvas  do
  begin
    // 角丸内を塗る
    if  not Transparent  then
      begin
        Pen.Style := psClear;
        Pen.Color := clWhite;   // psClear だけではプリンターによっては描いてしまう場合があるため
        Pie(X1 + XOffset, Y1 + YOffset, X2 + XOffset, Y2 + YOffset,
          X3 + XOffset, Y3 + YOffset, X4 + XOffset, Y4 + YOffset);
      end;
    // 角丸を書く
    if  Style <> lsNone  then
      begin
        Pen.Color := LineColor;
        SetLineStyle(Canvas, Style, PrintScale);
        Pen.Style := psSolid;
        Arc(X1, Y1, X2, Y2, X3, Y3, X4, Y4);
      end;
  end;
end;

procedure TLRItemR.PrintText(ACanvas: TCanvas; ARect: TRect;
  APrintScale: Double);
var
  SV_TextHeight: Integer;
  AText: string;
  Flags: Integer;
  CalcRect: TRect;        // Wordwrap時に印刷するテキストの高さを計算するためのRect
  PrintRect: TRect;       // 印刷する領域
begin
  ACanvas.Font := Self.Font;
  AText := Self.DisplayText;
  SV_TextHeight := ACanvas.TextHeight('|');
  // フラグのセット
  Flags := DT_NOPREFIX;
  if  WordWrap  then  Flags := Flags or DT_WORDBREAK
  else  Flags := Flags or DT_SINGLELINE;
  case  Alignment  of
    taLeftJustify: Flags := Flags or DT_LEFT;
    taRightJustify: Flags := Flags or DT_RIGHT;
    else  Flags := Flags or DT_CENTER;
  end;

  // 位置の計算
  with  ARect  do
    PrintRect := PrintScaleRect(Rect(Left, Top, Right-1, Bottom-1), APrintScale);
  with  CalcRect  do
  begin
    Top := 0;
    Bottom := SV_TextHeight;
    Left := 0;
    Right := PrintRect.Right - PrintRect.Left - ToPrintScale(HMargin, APrintScale);
  end;
  DrawText(ACanvas.Handle, PChar(AText), -1, CalcRect, Flags or DT_CALCRECT);
  SV_TextHeight := CalcRect.Bottom;

  // 横位置調整
  with  PrintRect  do
  case  Alignment  of
    taLeftJustify: Left := Left + ToPrintScale(HMargin, APrintScale);
    taRightJustify: Right := Right - ToPrintScale(HMargin, APrintScale);
  end;
  // 縦位置調整
  with  PrintRect  do
  case  Layout  of
    tlTop: Top := Top + ToPrintScale(VMargin, APrintScale);
    tlBottom: Top := Bottom - ToPrintScale(VMargin, APrintScale) - SV_TEXTHEIGHT;
    else
      Top := Top + (PrintRect.Bottom - PrintRect.Top - SV_TEXTHEIGHT) div 2;
  end;

  // 文字列の描写
  DrawText(ACanvas.Handle, PChar(AText), -1, PrintRect, Flags);
end;

function TLRItemR.DisplayText: string;
var
  s: string;
  i: Integer;
  flg: boolean;
begin
  // マスクが指定されていたら
  if  (DisplayMask <> '') and (Caption <> '')  then
    begin
      s := Caption;
      flg := True;
      for  i := 1 to Length(s)  do
        if not CharInSet(s[i], ['0'..'9', '.', '-', '+'])  then
          begin
            flg := False;
            Break;
          end;
      // 数字に変換できる場合はFormatFloatを使う
      if  flg  then
        Result := FormatFloat(DisplayMask, StrToFloat(Caption))
      else
        // 数字以外の場合はFormatMuskTextを使う
        Result := FormatMaskText(DisplayMask, Caption);
    end
  else
    Result := Caption;
end;

procedure TLRItemR.SetDiagonalRisedLeft(const Value: TLineStyle);
begin
  if  FDiagonalRisedLeft <> Value  then
    begin
      FDiagonalRisedLeft := Value;
      Invalidate;
    end;
end;

procedure TLRItemR.SetDiagonalRisedRight(const Value: TLineStyle);
begin
  if  FDiagonalRisedRight <> Value  then
    begin
      FDiagonalRisedRight := Value;
      Invalidate;
    end;
end;

end.
