﻿{ *
  *
  *   LightRepEx
  *
  *   TLRImegeコンポネント
  *   イメージを使用しないプログラムのファイルサイズを小さくするため
  *   LightRepから分離
  *
  *  Copyright(c) 1998-2001 Takezou  All rights reserved.
  *  Copyright(c) 2001-2003 Ohtaka(Kazuyoshi Ohta)  All rights reserved.
  *
  *  このコンポーネントは2001年9月より全ての著作権がTakezouからOhtakaに
  *  両者合意の上譲渡されました
  *
  *   99.01.18 Delphi4対応
  *   99.04.30 コンパイル指令を変更
  *   99.11.20 Stretch=falseが効かないバグを修正
  *   01.06.20 TLRImage.SetPictureメソッドのバグ修正
  *
}
{
  2001/10/26 TNkDIB(*.dib)のイメージに対応
}

unit LightRepEx;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, ExtCtrls,
  Printers, LightRep, JPEG, NkDIB, LRConsts, GIFImg, PNGImage;

{ TLRImage }
type
  {$IF CompilerVersion >= 23.0}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
  {$IFEND}
  TLRImage = class(TCustomLRItem)
  private
    FPicture: TPicture;
    FStretch: boolean;
    procedure SetPicture(Value: TPicture);
    procedure SetStretch(Value: boolean);
  protected
    procedure Paint; override;
    function CanAutoSize(var NewWidth, NewHeight: Integer): boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Print(ACanvas: TCanvas; Rect: TRect; APrintScale: Double;
      APrintStatus: TPrintStatus); override;
  published
    property AutoSize;
    property Picture: TPicture read FPicture write SetPicture;
    property Stretch: boolean read FStretch write SetStretch;
  end;

implementation

{ TLRImage }

constructor TLRImage.Create(AOwner: TComponent);
begin
  inherited;
  FPicture := TPicture.Create;
end;

procedure TLRImage.SetPicture(Value: TPicture);
begin
  // 有効なグラフィック以外の場合はエラー
  if (Value = nil) or
     (Value.Graphic = nil) or
     (Value.Graphic is TBitmap) or
     (Value.Graphic is TMetaFile) or
     (Value.Graphic is TJPEGImage) or
     (Value.Graphic is TGIFImage) or // D2007 or later
     (Value.Graphic is TPNGImage) or // D2009 or later
     {$IF CompilerVersion >= 21.0}
     (Value.Graphic is TWICImage) or // D2010 or later
     {$IFEND}
     (Value.Graphic is TNkDIB) then
    begin
      FPicture.Assign(Value);
      Invalidate;
    end
  else
    raise exception.Create(LRERR_INVALID_IMAGE);
end;

procedure TLRImage.SetStretch(Value: boolean);
begin
  if FStretch <> Value then
    begin
      FStretch := Value;
      Invalidate;
    end;
end;

function TLRImage.CanAutoSize(var NewWidth, NewHeight: Integer): boolean;
begin
  Result := True;
  if not (csDesigning in ComponentState) or (Picture.Width > 0) and
    (Picture.Height > 0) then
  begin
    if Align in [alNone, alLeft, alRight] then
      NewWidth := Picture.Width;
    if Align in [alNone, alTop, alBottom] then
      NewHeight := Picture.Height;
  end;
end;

procedure TLRImage.Paint;
begin
  // イメージがある場合はイメージを描画、無い場合は枠線を描画
  if (FPicture = nil) or (FPicture.Graphic = nil) or (FPicture.Graphic.Empty) then
    with Canvas do
      begin
        // コントロール名を描写する
        TextOut(4, 4, Name);
        // ペンを変更する
        Pen.Color := clGreen;
        Pen.Style := psDot;
        // 四角形を描写する
        MoveTo(0, 0);
        LineTo(Width - 1, 0);
        LineTo(Width - 1, Height - 1);
        LineTo(0, Height - 1);
        LineTo(0, 0);
      end
  else if FStretch then
    Canvas.StretchDraw(GetClientRect, FPicture.Graphic)
  else
    Canvas.Draw(0, 0, FPicture.Graphic)
end;

destructor TLRImage.Destroy;
begin
  FPicture.Free;
  inherited;
end;

procedure TLRImage.Print(ACanvas: TCanvas; Rect: TRect; APrintScale: Double;
  APrintStatus: TPrintStatus);
var
  Info: PBitmapInfo;
  InfoSize: Cardinal;
  ImageSize: Cardinal;
  Image: Pointer;
  DstRect: TRect;
begin
  // フラグがpsNormal以外の場合と、イメージが無い場合は何もしない
  if (APrintStatus <> psNormal) or
     (FPicture.Graphic = nil) or
     (FPicture.Graphic.Empty) then
    Exit;

  if (FPicture.Graphic is TMetaFile) then
    begin
      // サイズを印刷先のスケールに変換
      DstRect := PrintScaleRect(Rect, APrintScale);

      // [画像が Metafile の場合]
      // 直接出力する
      ACanvas.StretchDraw(DstRect, (FPicture.Graphic as TMetaFile));
    end
  else
    begin
      // [画像が Metafile 以外の場合]
      // 内部ビットマップに一旦グラフィックをコピー後、DIBに変換して出力する
      with TBitmap.Create do
        try
          Assign(FPicture.Graphic);

          // Strechがfalseの時は、サイズを調整する。
          if not FStretch then
            begin
              if Width > Rect.Right - Rect.Left then
                Width := Rect.Right - Rect.Left
              else
                Rect.Right := Width + Rect.Left;
              if Height > Rect.Bottom - Rect.Top then
                Height := Rect.Bottom - Rect.Top
              else
                Rect.Bottom := Height + Rect.Top;
            end;

          // サイズを印刷先のスケールに変換
          DstRect := PrintScaleRect(Rect, APrintScale);

          GetDIBSizes(Handle, InfoSize, ImageSize);
          Info := VirtualAlloc(nil, InfoSize, MEM_COMMIT or MEM_RESERVE, PAGE_READWRITE);
          try
            Image := VirtualAlloc(nil, ImageSize, MEM_COMMIT or MEM_RESERVE, PAGE_READWRITE);
            try
              GetDIB(Handle, Palette, Info^, Image^);
              with Info^.bmiHeader do
                begin
                  StretchDIBits(ACanvas.Handle,
                                DstRect.Left,
                                DstRect.Top,
                                DstRect.Right - DstRect.Left,
                                DstRect.Bottom - DstRect.Top,
                                0,
                                0,
                                biWidth,
                                biHeight,
                                Image,
                                Info^,
                                DIB_RGB_COLORS,
                                SRCCOPY);
                end;
            finally
              VirtualFree(Image, 0, MEM_RELEASE);
            end;
          finally
            VirtualFree(Info, 0, MEM_RELEASE);
          end;
        finally
          Free;
        end;
    end;
end;

end.
