﻿{*
 *
 *   LightRepEx2
 *
 *   TLRVariableTextコンポネント
 *   文字間隔・行間隔を自由に変更可能なテキストコンポーネント
 *
 *  Copyright(c) 1998-2001 Takezou  All rights reserved.
 *  Copyright(c) 2001-2003 Ohtaka(Kazuyoshi Ohta)  All rights reserved.
 *
 *  このコンポーネントは2001年9月より全ての著作権がTakezouからOhtakaに
 *  両者合意の上譲渡されました
 *
 *   2000.03.25 作成
 *   2000.03.28 一部修正
 *   2000.05.05 #13#10で強制改行した時に行間隔が指定した値にならないバグを修正
 *
 *   まだとりあえず動くというレベルなので、コードの整理が必要。
 *
 *
 *}
unit LightRepEx2;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  LightRep, LRConsts;

type
  {$IF CompilerVersion >= 23.0}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
  {$IFEND}  TLRVariableText = class(TCustomLRItem)
  private
    FCharSpace: integer;
    FWordSpace: integer;
    FWordwrap: boolean;
    FLeading: integer;
    FTransParent: boolean;
    procedure SetLeading(Value: integer);
    procedure SetCharSpace(Value: integer);
    procedure SetWordSpace(Value: integer);
    procedure SetWordwrap(Value: boolean);
    procedure SetTransParent(Value: boolean);
    procedure SetText(Value: string);
    function GetText: string;
    function InternalTextout(ACanvas: TCanvas; Rect: TRect;
                       APrintScale: Double; S: string; X, Y: integer): integer;
  protected
    procedure Paint; override;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure Print(ACanvas: TCanvas; ARect: TRect;
      APrintScale: double; APrintStatus: TPrintStatus); override;
  public
    function GetNextWord(const S: string; var Index: integer): string;
  published
    constructor Create(AOwner: TComponent); override;
    property Color;  
    property Font;
    property CharSpace: integer read FCharSpace write SetCharSpace;
    property Leading: integer read FLeading write SetLeading;
    property Text: string read GetText write SetText;
    property Transparent: boolean read FTransparent write SetTransparent;
    property WordSpace: integer read FWordSpace write SetWordSpace;
    property WordWrap: boolean read FWordWrap write SetWordwrap;
  end;

implementation

{ TLRVariableText }

procedure TLRVariableText.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
end;

function TLRVariableText.GetNextWord(const S: string;
  var Index: integer): string;
var
  ln: integer;
  i: integer;
begin
  result := '';
  ln := Length(S);
  if Index > ln then
    Exit;
  i := Index;
  while true do
    {*
     *  単語の区切り（スペース・改行・サロゲートペアの末尾・サロゲートエレメント間を探す）
     *
     *}
    if ((S[i] = #10) and (S[i-1] = #13)) or (S[i] = ' ') or
       (Pos(S[i], LIGHTREP_WRAPCHAR) > 0) then
      begin
        result := Copy(S, Index, i - (Index -1));
        break;
      end
    else if i >= ln then
      begin
        result := Copy(S, Index, i - (Index - 1));
        break;
      end
    else if ByteType(S, i) = mbTrailByte then
      begin
        result := Copy(S, Index, i - (Index - 1));
        break;
      end
    else if ((i < ln) and (ByteType(S, i + 1) = mbLeadByte)) then
      begin
        result := Copy(S, Index, i - (Index - 1));
        break;
      end
    else
      inc(i);

   Index := i + 1;

end;

function TLRVariableText.InternalTextout(ACanvas: TCanvas; Rect: TRect;
                       APrintScale: Double; S: string; X, Y: integer): integer;
var
  Pos: integer;
  i: integer;
  Word: string;
  ln: integer;
  flg: boolean;
begin
  i := 1;
  Pos := X;
  ln := Length(S);

  {*
   *  最後のスペースはWordSpaceに置き換えられる
   *
   *}
  flg := false;
  if (ln > 0) and (S[ln] = ' ') then
  begin
    flg := true;
    dec(ln);
    S := Copy(S, 1, ln);
  end
  else
  if ((ln >= 2) and (S[ln] = #10) and (S[ln-1] = #13)) then
  begin
    ln := ln - 2;
    S := Copy(S, 1, ln);
  end;

  while true do
  begin
    if i > ln then
      Break;
    if ByteType(S, i) = mbLeadByte then
    begin
      Word := Copy(S, i, 2);
      inc(i);
    end
    else
    begin
      Word := S[i];
    end;
    {*
     *   TextRectメソッドで一文字づつサイズを計算しつつ描画する。
     *   低速だが他に方法が浮かばないので。
     *
     *}
    ACanvas.TextRect(Rect, Pos, Y, Word);
    Pos := Pos + ACanvas.TextWidth(Word) + ToPrintScale(FCharSpace, APrintScale);
    inc(i);
  end;
  if Flg then
    result := Pos + ToPrintScale(FWordSpace, APrintScale)
  else
    result := Pos + ToPrintScale(FCharSpace, APrintScale);

end;

procedure TLRVariableText.Paint;
var
  TmpPrintScale: double;
begin
  TmpPrintScale := 1;
  Print(Canvas, GetClientRect, TmpPrintScale, psNormal);
  with Canvas do
  begin
    // ペンを変更する
    Pen.Color := clFuchsia;
    Pen.Style := psDot;
    // 四角形を描写する
    MoveTo(0, 0);
    LineTo(Width-1, 0);
    LineTo(Width-1, Height-1);
    LineTo(0, Height-1);
    LineTo(0, 0);
  end
end;

procedure TLRVariableText.Print(ACanvas: TCanvas; ARect: TRect;
  APrintScale: double; APrintStatus: TPrintStatus);
var
  i: integer;
  S: string;
  XPos, YPos: integer;
  tmpPos: integer;
  PrintRect: TRect;
  ln: integer;
begin
  if APrintStatus <> psNormal then
    Exit;

  with ACanvas do
  begin
    Brush.Color := Color;
    Font := Self.Font;

    // TmpRectを印刷面のスケールに変換する
    with ARect do
      PrintRect := PrintScaleRect(Rect(Left, Top, Right-1, Bottom-1), APrintScale);

    if FTransParent then
      Brush.Style := bsClear;

    FillRect(PrintRect);
    Brush.Style := bsClear;

    i := 1;
    XPos := PrintRect.Left;
    YPos := PrintRect.Top;

    while i <= Length(Text) do
    begin
      S := GetNextWord(Text, i);
      ln := Length(S);
      TmpPos := Xpos + ACanvas.TextWidth(S);
      if (FWordWrap and (TmpPos > PrintRect.Right) and (XPos <> PrintRect.Left)) then
      begin
        YPos := YPos - ACanvas.Font.Height + ToPrintScale(FLeading, APrintScale);
        XPos := PrintRect.Left;
      end;
      Xpos := InternalTextout(ACanvas, PrintRect, APrintScale, S, XPos, YPos);
      if (ln >= 2) and (S[ln-1]=#13) and (S[ln]=#10) and (XPos > PrintRect.Left) then
      begin
        YPos := YPos - ACanvas.Font.Height + ToPrintScale(FLeading, APrintScale);
        XPos := PrintRect.Left;
      end;
    end;
  end;
end;

procedure TLRVariableText.SetCharSpace(Value: integer);
begin
  if Value <> FCharSpace then
  begin
    FCharSpace := Value;
    Invalidate;
  end;
end;

procedure TLRVariableText.SetLeading(Value: integer);
begin
  if Value <> FLeading then
  begin
    FLeading := Value;
    Invalidate;
  end;
end;

procedure TLRVariableText.SetWordSpace(Value: integer);
begin
  if Value <> FWordSpace then
  begin
    FWordSpace := Value;
    Invalidate;
  end;
end;

procedure TLRVariableText.SetWordwrap(Value: boolean);
begin
  if Value <> FWordwrap then
  begin
    FWordwrap := Value;
    Invalidate;
  end;
end;

procedure TLRVariableText.SetTransParent(Value: boolean);
begin
  if FTransParent <> Value then
  begin
    FTransParent := Value;
    Invalidate;
  end;
end;

function TLRVariableText.GetText: string;
begin
  result := Caption;
end;

procedure TLRVariableText.SetText(Value: string);
begin
  if Caption <> Value then
    Caption := Value;
end;

constructor TLRVariableText.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FWordSpace := 5;
  FWordwrap := true;
  Transparent := true;
end;

end.
