﻿{*
 *  LightRepPost
 *
 *  郵政省のカスタマーバーコードを出力するアイテム
 *
 *  Copyright(c) 1998-2001 Takezou  All rights reserved.
 *  Copyright(c) 2001-2003 Ohtaka(Kazuyoshi Ohta)  All rights reserved.
 *
 *  このコンポーネントは2001年9月より全ての著作権がTakezouからOhtakaに
 *  両者合意の上譲渡されました
 *
 *  2000.03.24 一部修正（アルファベットが入っている場合の処理）
 *  2000.05.05 条件コンパイル文の修正（Delphi3でのインストールエラー回避）
 *
 *}

unit LightRepPost;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Imglist, LightRep, LRConsts;

const
  BAR_HAIHUN = 10;
  BAR_CC1 = 11;
  BAR_CC2 = 12;
  BAR_CC3 = 13;
  BAR_CC4 = 14;
  BAR_CC5 = 15;
  BAR_CC6 = 16;
  BAR_CC7 = 17;
  BAR_CC8 = 18;
  BAR_STR = 19;
  BAR_END = 20;

type
  {$IF CompilerVersion >= 23.0}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
  {$IFEND}
  TLRCustBarItem = class(TCustomLRItem)
  private
    FCode: string;
    FImageList: TImageList;
    procedure SetCode(Value: string);
  protected
    procedure Paint; override;
    procedure Print(ACanvas: TCanvas; ARect: TRect;
      APrintScale: double; APrintStatus: TPrintStatus); override;
    procedure StretchDIBDraw(ACanvas: TCanvas; Rect: TRect; Bitmap: TBitmap);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Code: string read FCode write SetCode;
  end;

implementation

{$R PostBarcode.res}

procedure TLRCustBarItem.SetCode(Value: string);
const
  ValidChars = ['0'..'9', '-', 'A'..'Z'];
var
  i: integer;
begin
  Value := Trim(UpperCase(Value));
  for i := 1 to Length(Value) do
    if not CharInSet(Value[i], ValidChars) then
      raise Exception.Create(Format(LRERR_INVALID_CHARCTER, [Value[i]]));
  FCode := Value;
  Invalidate;
end;

procedure TLRCustBarItem.Print(ACanvas: TCanvas; ARect: TRect;
      APrintScale: double; APrintStatus: TPrintStatus);
var
  i, j: integer;
  FSize: integer;
  FBitmap: TBitmap;
  FRect: TRect;
  FLen: integer;
  FDigit: integer;
  FCodeArray: array[0..22] of Byte;
  TmpNumber: Byte;
const
  INTEGERS = ['0'..'9'];
  ALPHABETS = ['A'..'Z'];
begin
  FBitmap := TBitmap.Create;
  FRect := PrintScaleRect(ARect, APrintScale);
  FSize := Trunc(Height*APrintScale);

  FRect.Bottom := FRect.Top + FSize;

  FLen := Length(FCode);
  FCodeArray[0] := BAR_STR;
  FCodeArray[22] := BAR_END;
  i := 1;
  j := 1;
  while i <= 20 do
  begin
    if j > FLen then
      FCodeArray[i] := BAR_CC4
    else
    if CharInSet(FCode[j], INTEGERS) then
      FCodeArray[i] := StrToInt(FCode[j])
    else
    if FCode[j] = '-' then
      FCodeArray[i] := BAR_HAIHUN
    else
    begin
      {*
       * アルファベットの場合の処理
       * アルファベットはAからJがそれぞれ"CC1"+"0"～"9"
       * KからTがそれぞれ"CC2"+"0"～"9"、UからZがそれぞれ"CC2"+"0"～"5"つまり
       * CC1～3と数字の組み合わせで表現される。
       *
       *}
      TmpNumber := Ord(FCode[j]) - 65;
      FCodeArray[i] := TmpNumber div 10 + 11;
      inc(i);
      if i <= 20 then
        FCodeArray[i] := TmpNumber mod 10;
    end;

    inc(i);
    inc(j);
  end;

  {* チェックデジットの計算 *}
  FDigit := 0;
  for i := 0 to 20 do
    FDigit := FDigit + FCodeArray[i];
  FCodeArray[21] := 19 - (FDigit mod 19);
  if FCodeArray[21] = 19 then FCodeArray[21] := 0;

  for i := 0 to 22 do
  begin
    {*
     *  イメージリストから、バーコードビットマップを取得する
     *
     *}
    FBitmap.Canvas.Brush.Color := clWhite;
    FBitmap.Canvas.FillRect(Rect(0, 0, FBitmap.Width, FBitmap.Height));
    FImageList.GetBitmap(FCodeArray[i], FBitmap);

    {*
     *  バーコードビットマップを適当な位置に出力
     *
     *}
    FRect.Right := FRect.Left + FSize;
    StretchDIBDraw(ACanvas, FRect, FBitmap);
    FRect.Left := FRect.Left + FSize;
  end;

  FBitmap.Free;
end;

procedure TLRCustBarItem.Paint;
begin
  // コードが指定されている場合はバーコードイメージを描画、無い場合は枠線を描画
  if FCode = '' then
    with Canvas do
    begin
      // コントロール名を描写する
      TextOut(4, 4, Name);
      // ペンを変更する
      Pen.Color := clRed;
      Pen.Style := psDot;
      // 四角形を描写する
      MoveTo(0, 0);
      LineTo(Width-1, 0);
      LineTo(Width-1, Height-1);
      LineTo(0, Height-1);
      LineTo(0, 0);
    end
  else
    Print(Canvas, GetClientRect, 1, psNormal);
end;

procedure TLRCustBarItem.StretchDIBDraw(ACanvas: TCanvas; Rect: TRect; Bitmap: TBitmap);
var
  Info: PBitmapInfo;
  InfoSize: Cardinal;
  ImageSize: Cardinal;
  Image: Pointer;
  DstRect: TRect;
begin
  DstRect := Rect;
  with Bitmap do
  begin
    GetDIBSizes(Handle, InfoSize, ImageSize);
    Info := VirtualAlloc(nil, InfoSize,
      MEM_COMMIT or MEM_RESERVE, PAGE_READWRITE);
    try
      Image := VirtualAlloc(nil, ImageSize,
        MEM_COMMIT or MEM_RESERVE, PAGE_READWRITE);
      try
        GetDIB(Handle, Palette, Info^, Image^);
        with Info^.bmiHeader do
        begin
          StretchDIBits(ACanvas.Handle, DstRect.Left, DstRect.Top,
            DstRect.Right - DstRect.Left, DstRect.Bottom - DstRect.Top,
            0, 0, biWidth, biHeight, Image, Info^, DIB_RGB_COLORS, SRCCOPY);
        end;
      finally
        VirtualFree(Image, 0, MEM_RELEASE);
      end;
    finally
      VirtualFree(Info, 0, MEM_RELEASE);
    end;
  end;
end;

constructor TLRCustBarItem.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Height := 12;
  Width := 276;
  FImageList := TImageList.Create(Self);
  FImageList.Width := 12;
  FImageList.Height := 12;
  if not FImageList.ResInstLoad(HInstance, rtBitmap, 'POSTBAR', clWhite) then
    raise Exception.Create(LRERR_FAILED_LOAD_RESOURCE);
end;

destructor TLRCustBarItem.Destroy;
begin
  FImageList.Free;
  inherited;
end;

end.
 